# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock
from unittest.mock import call

from osc_lib import exceptions

from otcextensions.osclient.nat.v2 import dnat
from otcextensions.tests.unit.osclient.nat.v2 import fakes

from openstackclient.tests.unit import utils as tests_utils


class TestListDnatRules(fakes.TestNat):

    objects = fakes.FakeDnatRule.create_multiple(3)

    column_list_headers = (
        'Id',
        'Nat Gateway Id',
        'Port Id',
        'Private IP',
        'Floating Ip Address',
        'Protocol',
        'Status'
    )
    columns = (
        'id',
        'nat_gateway_id',
        'port_id',
        'private_ip',
        'floating_ip_address',
        'protocol',
        'status'
    )

    data = []

    for s in objects:
        data.append((
            s.id,
            s.nat_gateway_id,
            s.port_id,
            s.private_ip,
            s.floating_ip_address,
            s.protocol,
            s.status
        ))

    def setUp(self):
        super(TestListDnatRules, self).setUp()

        self.cmd = dnat.ListDnatRules(self.app, None)

        self.client.dnat_rules = mock.Mock()
        self.client.api_mock = self.client.dnat_rules

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            '--limit', '1',
            '--id', '2',
            '--nat-gateway-id', '3',
            '--project-id', '4',
            '--private-ip', '5',
            '--internal-service-port', '6',
            '--protocol', '7',
            '--floating-ip-id', '8',
            '--floating-ip-address', '9',
            '--admin-state-up', '10',
            '--created-at', '11',
            '--status', '12'
        ]

        verifylist = [
            ('limit', 1),
            ('id', '2'),
            ('nat_gateway_id', '3'),
            ('project_id', '4'),
            ('private_ip', '5'),
            ('internal_service_port', '6'),
            ('protocol', '7'),
            ('floating_ip_id', '8'),
            ('floating_ip_address', '9'),
            ('admin_state_up', '10'),
            ('created_at', '11'),
            ('status', '12'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            limit=1,
            id='2',
            nat_gateway_id='3',
            project_id='4',
            private_ip='5',
            internal_service_port='6',
            protocol='7',
            floating_ip_id='8',
            floating_ip_address='9',
            admin_state_up='10',
            created_at='11',
            status='12',
        )


class TestShowDnatRule(fakes.TestNat):

    _data = fakes.FakeDnatRule.create_one()

    columns = (
        'admin_state_up',
        'created_at',
        'external_service_port',
        'floating_ip_address',
        'floating_ip_id',
        'id',
        'internal_service_port',
        'nat_gateway_id',
        'port_id',
        'private_ip',
        'project_id',
        'protocol',
        'status'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestShowDnatRule, self).setUp()

        self.cmd = dnat.ShowDnatRule(self.app, None)

        self.client.get_dnat_rule = mock.Mock(return_value=self._data)

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        # Testing that a call without the required argument will fail and
        # throw a "ParserExecption"
        self.assertRaises(tests_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_show(self):
        arglist = [
            self._data.id,
        ]

        verifylist = [
            ('dnat', self._data.id),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.get_dnat_rule.assert_called_with(self._data.id)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_show_non_existent(self):
        arglist = [
            'unexist_dnat_rule_id',
        ]

        verifylist = [
            ('dnat', arglist[0]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = exceptions.CommandError('Resource Not Found')
        self.client.get_dnat_rule = (
            mock.Mock(side_effect=find_mock_result)
        )

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('Resource Not Found', str(e))
        self.client.get_dnat_rule.assert_called_with(arglist[0])


class TestCreateDnatRule(fakes.TestNat):

    _data = fakes.FakeDnatRule.create_one()

    columns = (
        'admin_state_up',
        'created_at',
        'external_service_port',
        'floating_ip_address',
        'floating_ip_id',
        'id',
        'internal_service_port',
        'nat_gateway_id',
        'port_id',
        'private_ip',
        'project_id',
        'protocol',
        'status'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestCreateDnatRule, self).setUp()

        self.cmd = dnat.CreateDnatRule(self.app, None)

        self.client.create_dnat_rule = mock.Mock(return_value=self._data)

    def test_create(self):
        arglist = [
            '--nat-gateway-id', 'test-nat-uuid',
            '--floating-ip-id', 'test-floating-ip-uuid',
            '--protocol', 'tcp',
            '--internal-service-port', '80',
            '--external-service-port', '80',
            '--private-ip', '192.168.0.99',
        ]

        verifylist = [
            ('nat_gateway_id', 'test-nat-uuid'),
            ('floating_ip_id', 'test-floating-ip-uuid'),
            ('protocol', 'tcp'),
            ('internal_service_port', '80'),
            ('external_service_port', '80'),
            ('private_ip', '192.168.0.99'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.create_dnat_rule.assert_called_with(
            nat_gateway_id='test-nat-uuid',
            floating_ip_id='test-floating-ip-uuid',
            protocol='tcp',
            internal_service_port='80',
            external_service_port='80',
            private_ip='192.168.0.99')

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteDnatRule(fakes.TestNat):

    _data = fakes.FakeDnatRule.create_multiple(2)

    def setUp(self):
        super(TestDeleteDnatRule, self).setUp()

        self.client.delete_dnat_rule = mock.Mock(return_value=None)

        self.cmd = dnat.DeleteDnatRule(self.app, None)

    def test_delete(self):
        arglist = [
            self._data[0].id,
        ]

        verifylist = [
            ('dnat', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.client.get_dnat_rule = (
            mock.Mock(return_value=self._data[0])
        )

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.get_dnat_rule.assert_called_with(self._data[0].id)
        self.client.delete_dnat_rule.assert_called_with(self._data[0].id)
        self.assertIsNone(result)

    def test_multiple_delete(self):
        arglist = []

        for dnat_rule in self._data:
            arglist.append(dnat_rule.id)

        verifylist = [
            ('dnat', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = self._data
        self.client.get_dnat_rule = (
            mock.Mock(side_effect=find_mock_result)
        )

        # Trigger the action
        result = self.cmd.take_action(parsed_args)

        calls = []
        for dnat_rule in self._data:
            calls.append(call(dnat_rule.id))
        self.client.delete_dnat_rule.assert_has_calls(calls)
        self.assertIsNone(result)

    def test_multiple_delete_with_exception(self):
        arglist = [
            self._data[0].id,
            'unexist_dnat_rule_id',
        ]
        verifylist = [
            ('dnat', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = [self._data[0], exceptions.CommandError]
        self.client.get_dnat_rule = (
            mock.Mock(side_effect=find_mock_result)
        )

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('1 of 2 DNAT Rule(s) failed to delete.', str(e))

        self.client.get_dnat_rule.assert_any_call(arglist[0])
        self.client.get_dnat_rule.assert_any_call(arglist[1])
        self.client.delete_dnat_rule.assert_called_once_with(arglist[0])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class DimensionsSpec(resource.Resource):

    # Properties
    #: dimension.name: object type e.g. ECS (instance_id)
    name = resource.Body('name')
    #: dimension.value: object id e.g. ECS ID
    value = resource.Body('value')


class Metric(resource.Resource):
    resource_key = 'metric'
    resources_key = 'metrics'
    base_path = '/metrics'

    # capabilities
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'dim', 'limit', 'metric_name', 'namespace', 'order', 'start'
    )

    # Properties
    #: List of metric dimensions
    #: dimension.name: object type e.g. ECS (instance_id)
    #: dimension.value: object id e.g. ECS ID
    dimensions = resource.Body('dimensions', type=list,
                               list_type=DimensionsSpec)
    #: Shows the metric name
    metric_name = resource.Body('metric_name')
    #: Indicates the metric namespaces
    namespace = resource.Body('namespace')
    #: Indicates the metric unit
    unit = resource.Body('unit')



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import mock
# from osc_lib import utils as common_utils

from otcextensions.osclient.rds.v1 import flavor
from otcextensions.tests.unit.osclient.rds.v1 import fakes as rds_fakes


class TestListDatabaseFlavors(rds_fakes.TestRds):

    column_list_headers = [
        'ID',
        'Name',
        'ram',
        'spec_code'
        # 'Str_ID',
        # 'vCPUs'
    ]

    def setUp(self):
        super(TestListDatabaseFlavors, self).setUp()

        self.cmd = flavor.ListDatabaseFlavors(self.app, None)

        self.app.client_manager.rds.flavors = mock.Mock()

        self.flavors = self.flavor_mock.create_multiple(3)
        self.flavor_data = []
        for s in self.flavors:
            self.flavor_data.append((
                s.id,
                s.name,
                s.ram,
                s.spec_code
            ))

    def test_list_flavors(self):
        arglist = [
            'dbId',
            'regio'
        ]

        verifylist = [
            ('dbId', 'dbId'),
            ('region', 'regio')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.app.client_manager.rds.flavors.side_effect = [
            self.flavors
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.app.client_manager.rds.flavors.assert_called()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(tuple(self.flavor_data), tuple(data))

    def test_list_flavors_default_region(self):
        arglist = [
            'dbId',
        ]

        verifylist = [
            ('dbId', 'dbId'),
            ('region', 'eu-de')
        ]

        # Verify cm is triggereg with default parameters
        self.check_parser(self.cmd, arglist, verifylist)


class TestShowDatabaseFlavors(rds_fakes.TestRds):

    columns = ('ID', 'Name', 'ram', 'spec_code')

    def setUp(self):
        super(TestShowDatabaseFlavors, self).setUp()

        self.cmd = flavor.ShowDatabaseFlavor(self.app, None)

        self.app.client_manager.rds.get_flavor = mock.Mock()

        self.flavor = self.flavor_mock.create_one()

        self.flavor_data = (
            self.flavor.id,
            self.flavor.name,
            self.flavor.ram,
            self.flavor.spec_code
        )

    def test_show_flavor(self):
        arglist = [
            'test_flavor',
        ]

        verifylist = [
            ('flavor', 'test_flavor'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.app.client_manager.rds.get_flavor.side_effect = [
            self.flavor
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.app.client_manager.rds.get_flavor.assert_called()

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.flavor_data, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""VPC Endpoint Service v1 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {}
    hidden = [
        'location',
    ]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden
    )


class ListQuota(command.Lister):

    _description = _('List VPC endpoint resource quotas.')
    columns = (
        'Type',
        'Quota',
        'Used',
    )

    def get_parser(self, prog_name):
        parser = super(ListQuota, self).get_parser(prog_name)
        parser.add_argument(
            '--type',
            metavar='{endpoint, endpoint_service}',
            type=lambda s: s.lower(),
            choices=['endpoint', 'endpoint_service'],
            help=_('Specify the resource type.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.vpcep
        data = client.resource_quota(getattr(parsed_args, 'type'))

        return (
            self.columns,
            (utils.get_item_properties(s, self.columns) for s in data),
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import requests
from unittest import mock

from otcextensions.sdk.identity.v3 import _proxy
from otcextensions.sdk.identity.v3 import credential
from otcextensions.sdk.identity.v3 import custom_role
from otcextensions.sdk.identity.v3 import agency
from otcextensions.sdk.identity.v3 import agency_role

from openstack.tests.unit import test_proxy_base


class FakeResponse:
    def __init__(self, response, status_code=200, headers=None):
        self.body = response
        self.status_code = status_code
        headers = headers if headers else {'content-type': 'application/json'}
        self.headers = requests.structures.CaseInsensitiveDict(headers)

    def json(self):
        return self.body


class TestIdentityProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestIdentityProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestIdentityCredential(TestIdentityProxy):
    def test_credential_create(self):
        self.verify_create(self.proxy.create_credential, credential.Credential,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id',
                                            'prepend_key': True})

    def test_credential_delete(self):
        self.verify_delete(self.proxy.delete_credential,
                           credential.Credential, True)

    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_alternate_endpoint',
        return_value='fake')
    def test_credential_find(self, epo_mock):
        self.verify_find(self.proxy.find_credential, credential.Credential)

    def test_credential_get(self):
        self.verify_get(self.proxy.get_credential, credential.Credential)

    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_alternate_endpoint',
        return_value='fake')
    def test_credentials(self, epo_mock):
        self.verify_list(
            self.proxy.credentials,
            credential.Credential,
        )
        epo_mock.assert_called_with()

    def test_credential_update(self):
        self.verify_update(self.proxy.update_credential, credential.Credential)


class TestIdentityAgency(TestIdentityProxy):
    def test__agencynormalize_domain(self):
        with mock.patch.object(self.proxy, 'session') as s_mock:
            access_mock = mock.Mock()
            access_mock.domain_id = 'fake_domain'
            s_mock.auth = mock.Mock()
            s_mock.auth.get_auth_ref = mock.Mock(
                return_value=access_mock
            )

            self.assertDictEqual({
                'domain_id': 'fake_domain'
            }, self.proxy._agency_normalize_domain_id()
            )

    def test_agency_create(self):
        self.verify_create(self.proxy.create_agency, agency.Agency,
                           method_kwargs={'name': 'id', 'domain_id': 'fake'},
                           expected_kwargs={'name': 'id',
                                            'domain_id': 'fake',
                                            'prepend_key': True})

    def test_agency_delete(self):
        self.verify_delete(self.proxy.delete_agency,
                           agency.Agency, True)

    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_alternate_endpoint',
        return_value='fake')
    def test_agency_find(self, epo_mock):
        self.verify_find(
            self.proxy.find_agency, agency.Agency,
            method_kwargs={'domain_id': 'fake'},
            expected_kwargs={'domain_id': 'fake'}
        )

    def test_agency_get(self):
        self.verify_get(self.proxy.get_agency, agency.Agency)

    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_alternate_endpoint',
        return_value='fake')
    def test_agencies(self, epo_mock):
        self.verify_list(
            self.proxy.agencies,
            agency.Agency,
            method_kwargs={'domain_id': 'fake'},
            expected_kwargs={'domain_id': 'fake'}
        )
        epo_mock.assert_called_with()

    def test_agency_update(self):
        self.verify_update(self.proxy.update_agency, agency.Agency)


class TestIdentityAgencyProjectRoles(TestIdentityProxy):

    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_resource',
        return_value=agency.Agency(id='fake'))
    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_alternate_endpoint',
        return_value='fake')
    def test_agency_project_roles(self, epo_mock, gr_mock):
        self.verify_list(
            self.proxy.agency_project_roles,
            agency_role.AgencyRole,
            method_kwargs={
                'agency': 'fake_agency',
                'project_id': 'fake_project'
            },
            expected_kwargs={
                'agency_id': 'fake',
                'role_ref_type': 'project',
                'role_ref_id': 'fake_project'
            }
        )

    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_resource')
    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_alternate_endpoint',
        return_value='fake')
    def test_check_agency_project_role(self, epo_mock, gr_mock):
        agency_fake = agency.Agency(id='fake')
        agency_role_fake = agency_role.AgencyRole(id='fake')
        gr_mock.side_effect = [
            agency_fake,
            agency_role_fake
        ]
        expected_calls = [
            mock.call(agency.Agency, 'fake_agency'),
            mock.call(
                agency_role.AgencyRole,
                {'agency_id': 'fake',
                 'role_ref_type': 'project',
                 'role_ref_id': 'fake_project',
                 'id': 'fake_role'})
        ]
        self._verify(
            'openstack.proxy.Proxy._head',
            self.proxy.check_agency_project_role,
            method_kwargs={
                'agency': 'fake_agency',
                'project_id': 'fake_project',
                'role_id': 'fake_role'
            },
            expected_args=[
                agency_role.AgencyRole,
                agency_role_fake
            ],
        )
        gr_mock.assert_has_calls(expected_calls)

    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_resource')
    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_alternate_endpoint',
        return_value='fake')
    def test_grant_agency_project_role(self, epo_mock, gr_mock):
        agency_fake = agency.Agency(id='fake')
        agency_role_fake = agency_role.AgencyRole(
            id='fake',
            role_ref_type='project',
            role_ref_id='fake_project',
            agency_id='fake')
        gr_mock.side_effect = [
            agency_fake,
            agency_role_fake,
        ]
        self._verify(
            'openstack.proxy.Proxy.put',
            self.proxy.grant_agency_project_role,
            method_kwargs={
                'agency': 'fake_agency',
                'project_id': 'fake_project',
                'role_id': 'fake_role'
            },
            expected_args=[
                'v3.0/OS-AGENCY/projects/fake_project/agencies/fake/roles/fake'
            ],
        )

    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_resource')
    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_alternate_endpoint',
        return_value='fake')
    def test_revoke_agency_project_role(self, epo_mock, gr_mock):
        agency_fake = agency.Agency(id='fake')
        agency_role_fake = agency_role.AgencyRole(
            id='fake',
            role_ref_type='project',
            role_ref_id='fake_project',
            agency_id='fake')
        gr_mock.side_effect = [
            agency_fake,
            agency_role_fake,
        ]
        self._verify(
            'openstack.proxy.Proxy._delete',
            self.proxy.revoke_agency_project_role,
            method_kwargs={
                'agency': 'fake_agency',
                'project_id': 'fake_project',
                'role_id': 'fake_role'
            },
            expected_args=[
                agency_role.AgencyRole, agency_role_fake
            ],
        )


class TestIdentityAgencyDomainRoles(TestIdentityProxy):

    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_resource',
        return_value=agency.Agency(id='fake'))
    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_alternate_endpoint',
        return_value='fake')
    def test_agency_domain_roles(self, epo_mock, gr_mock):
        self.verify_list(
            self.proxy.agency_domain_roles,
            agency_role.AgencyRole,
            method_kwargs={
                'agency': 'fake_agency',
                'domain_id': 'fake_domain'
            },
            expected_kwargs={
                'agency_id': 'fake',
                'role_ref_type': 'domain',
                'role_ref_id': 'fake_domain'
            }
        )

    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_resource')
    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_alternate_endpoint',
        return_value='fake')
    def test_check_agency_domain_role(self, epo_mock, gr_mock):
        agency_fake = agency.Agency(id='fake')
        agency_role_fake = agency_role.AgencyRole(id='fake')
        gr_mock.side_effect = [
            agency_fake,
            agency_role_fake
        ]
        expected_calls = [
            mock.call(agency.Agency, 'fake_agency'),
            mock.call(
                agency_role.AgencyRole,
                {'agency_id': 'fake',
                 'role_ref_type': 'domain',
                 'role_ref_id': 'fake_domain',
                 'id': 'fake_role'})
        ]
        self._verify(
            'openstack.proxy.Proxy._head',
            self.proxy.check_agency_domain_role,
            method_kwargs={
                'agency': 'fake_agency',
                'domain_id': 'fake_domain',
                'role_id': 'fake_role'
            },
            expected_args=[
                agency_role.AgencyRole,
                agency_role_fake
            ],
        )
        gr_mock.assert_has_calls(expected_calls)

    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_resource')
    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_alternate_endpoint',
        return_value='fake')
    def test_grant_agency_domain_role(self, epo_mock, gr_mock):
        agency_fake = agency.Agency(id='fake')
        agency_role_fake = agency_role.AgencyRole(
            id='fake',
            role_ref_type='domain',
            role_ref_id='fake_domain',
            agency_id='fake')
        gr_mock.side_effect = [
            agency_fake,
            agency_role_fake,
        ]
        self._verify(
            'openstack.proxy.Proxy.put',
            self.proxy.grant_agency_domain_role,
            method_kwargs={
                'agency': 'fake_agency',
                'domain_id': 'fake_domain',
                'role_id': 'fake_role'
            },
            expected_args=[
                'v3.0/OS-AGENCY/domains/fake_domain/agencies/fake/roles/fake'
            ],
        )

    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_resource')
    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_alternate_endpoint',
        return_value='fake')
    def test_revoke_agency_domain_role(self, epo_mock, gr_mock):
        agency_fake = agency.Agency(id='fake')
        agency_role_fake = agency_role.AgencyRole(
            id='fake',
            role_ref_type='domain',
            role_ref_id='fake_domain',
            agency_id='fake')
        gr_mock.side_effect = [
            agency_fake,
            agency_role_fake,
        ]
        self._verify(
            'openstack.proxy.Proxy._delete',
            self.proxy.revoke_agency_domain_role,
            method_kwargs={
                'agency': 'fake_agency',
                'domain_id': 'fake_domain',
                'role_id': 'fake_role'
            },
            expected_args=[
                agency_role.AgencyRole, agency_role_fake
            ],
        )


class TestIdentitySecurityTokens(TestIdentityProxy):

    @mock.patch('otcextensions.sdk.identity.v3._proxy.Proxy.post')
    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_alternate_endpoint',
        return_value='fake')
    def test_create_security_token_token(self, elt_ep_mock, post_mock):
        post_mock.return_value = FakeResponse({
            'credential': {'access': 'f1'}}
        )
        token = self.proxy.create_security_token(duration=2, method='token')
        post_mock.assert_called_with(
            'fake/v3.0/OS-CREDENTIAL/securitytokens',
            json={'auth': {'identity': {
                'methods': ['token'], 'token': {'duration-secods': 2}}}}
        )
        self.assertEqual('f1', token.access)

    @mock.patch('otcextensions.sdk.identity.v3._proxy.Proxy.post')
    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_alternate_endpoint',
        return_value='fake')
    def test_create_security_token_agency(self, elt_ep_mock, post_mock):
        post_mock.return_value = FakeResponse({
            'credential': {'access': 'f1'}}
        )
        token = self.proxy.create_security_token(
            duration=2, method='assume_role',
            domain_id='d1', xrole_name='xr1'
        )
        post_mock.assert_called_with(
            'fake/v3.0/OS-CREDENTIAL/securitytokens',
            json={'auth': {'identity': {
                'methods': ['assume_role'], 'assume_role': {
                    'domain_id': 'd1', 'xrole_name': 'xr1',
                    'duration-secods': 2}}}}
        )
        self.assertEqual('f1', token.access)


class TestIdentityCustomRole(TestIdentityProxy):
    @mock.patch(
        'otcextensions.sdk.identity.v3._proxy.Proxy._get_alternate_endpoint',
        return_value='fake')
    def test_custom_roles(self, epo_mock):
        self.verify_list(
            self.proxy.custom_roles,
            custom_role.CustomRole,
        )
        epo_mock.assert_called_with()



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy

from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.cce.v3 import node_pool

EXAMPLE_LIST = {
    "kind": "List",
    "apiVersion": "v3",
    "items": [
        {
            "kind": "NodePool",
            "apiVersion": "v3",
            "metadata": {
                "name": "wyr-17-nodepool-53042",
                "uid": "feec6013-cd7e-11ea-8c7a-0255ac100be7"
            },
            "spec": {
                "initialNodeCount": 0,
                "type": "vm",
                "nodeTemplate": {
                    "flavor": "s6.large.2",
                    "az": "eu-de-01",
                    "os": "EulerOS 2.5",
                    "login": {
                        "sshKey": "KeyPair-nodepool",
                        "userPassword": {}
                    },
                    "rootVolume": {
                        "volumetype": "SATA",
                        "size": 40
                    },
                    "dataVolumes": [
                        {
                            "volumetype": "SATA",
                            "size": 100,
                            "extendParam": {
                                "useType": "docker"
                            }
                        }
                    ],
                    "publicIP": {
                        "eip": {
                            "bandwidth": {}
                        }
                    },
                    "nodeNicSpec": {
                        "primaryNic": {
                            "subnetId": "31be174a-0c7f-4b71-bb0d"
                        }
                    },
                    "billingMode": 0,
                    "taints": [
                        {
                            "key": "aaa",
                            "value": "bbb",
                            "effect": "NoSchedule"
                        },
                        {
                            "key": "ccc",
                            "value": "ddd",
                            "effect": "NoSchedule"
                        }
                    ],
                    "k8sTags": {
                        "cce.cloud.com/cce-nodepool": "wyr-17-nodepool",
                        "tag1": "value1",
                        "tag2": "value2"
                    },
                    "userTags": [
                        {
                            "key": "resource-tag1",
                            "value": "value1"
                        },
                        {
                            "key": "resource-tag2",
                            "value": "value2"
                        }
                    ],
                    "extendParam": {
                        "DockerLVMConfigOverride": "dockerThinpool",
                        "alpha.cce/NodeImageID": "85bd7ec5",
                        "alpha.cce/postInstall": "bHMgLWwK",
                        "alpha.cce/preInstall": "bHMgLWw=",
                        "maxPods": 110,
                        "publicKey": "ssh-key"
                    }
                },
                "autoscaling": {
                    "enable": True,
                    "minNodeCount": 1,
                    "maxNodeCount": 3,
                    "scaleDownCooldownTime": 10,
                    "priority": 1
                },
                "nodeManagement": {
                    "serverGroupReference": "grpref"
                }
            },
            "status": {
                "currentNode": 0,
                "phase": ""
            }
        }
    ]
}

EXAMPLE_CREATE = {
    "kind": "NodePool",
    "apiVersion": "v3",
    "metadata": {
        "name": "wyr-17-nodepool-53042"
    },
    "spec": {
        "initialNodeCount": 0,
        "type": "vm",
        "autoscaling": {
            "enable": True,
            "minNodeCount": 1,
            "maxNodeCount": 3,
            "scaleDownCooldownTime": 10,
            "priority": 1
        },
        "nodeManagement": {
            "serverGroupReference": "grpref"
        },
        "nodeTemplate": {
            "flavor": "s6.large.2",
            "az": "eu-de-01",
            "os": "EulerOS 2.5",
            "login": {
                "sshKey": "KeyPair-nodepool"
            },
            "rootVolume": {
                "volumetype": "SATA",
                "size": 40
            },
            "dataVolumes": [
                {
                    "volumetype": "SATA",
                    "size": 100,
                    "extendParam": {
                        "useType": "docker"
                    }
                }
            ],
            "billingMode": 0,
            "extendParam": {
                "alpha.cce/preInstall": "bHMgLWw=",
                "alpha.cce/postInstall": "bHMgLWwK",
                "alpha.cce/NodeImageID": "85bd7ec5",
                "maxPods": 110,
                "DockerLVMConfigOverride": "dockerThinpool"
            },
            "k8sTags": {
                "tag1": "value1",
                "tag2": "value2"
            },
            "taints": [
                {
                    "key": "aaa",
                    "value": "bbb",
                    "effect": "NoSchedule"
                },
                {
                    "key": "ccc",
                    "value": "ddd",
                    "effect": "NoSchedule"
                }
            ],
            "userTags": [
                {
                    "key": "resource-tag1",
                    "value": "value1"
                },
                {
                    "key": "resource-tag2",
                    "value": "value2"
                }
            ],
            "nodeNicSpec": {
                "primaryNic": {
                    "subnetId": "31be174a-0c7f-4b71-bb0d"
                }
            }
        }
    }
}


class TestNodePool(base.TestCase):

    def setUp(self):
        super(TestNodePool, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.default_microversion = '1'
        self.sess._get_connection = mock.Mock(return_value=self.cloud)

        self.sot = node_pool.NodePool(cluster_id='cluster_id')

    def test_basic(self):
        sot = node_pool.NodePool()
        self.assertEqual(None, sot.resource_key)
        self.assertEqual(None, sot.resources_key)
        self.assertEqual('/clusters/%(cluster_id)s/nodepools',
                         sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        obj = EXAMPLE_LIST['items'][0]
        sot = node_pool.NodePool.existing(**obj)
        self.assertEqual(obj['metadata']['uid'], sot.id)
        self.assertEqual(obj['kind'], sot.kind)

    def test_list(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = copy.deepcopy(EXAMPLE_LIST)

        self.sess.get.return_value = mock_response

        result = list(
            self.sot.list(
                self.sess,
                cluster_id='cluster_id',
            )
        )

        self.sess.get.assert_called_once_with(
            '/clusters/%s/nodepools' % self.sot.cluster_id,
            params={},
            headers={
                'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
        )

        expected_list = [
            node_pool.NodePool.existing(
                **EXAMPLE_LIST['items'][0]),
        ]

        self.assertEqual(
            set(i.id for i in expected_list),
            set(i.id for i in result))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import api
from otcextensions.sdk.apig.v2 import api_supplements as _as

EXAMPLE = {
    "group_id": "id",
    "name": "test_api_001",
    "auth_type": "IAM",
    "backend_type": "HTTP",
    "req_protocol": "HTTP",
    "req_uri": "/test/http",
    "remark": "Mock backend API",
    "type": 2,
    "req_method": "GET",
    "result_normal_sample": "Example success response",
    "result_failure_sample": "Example failure response",
    "tags": ["httpApi"],
    "backend_api": {
        "req_protocol": "HTTP",
        "req_method": "GET",
        "req_uri": "/test/benchmark",
        "timeout": 5000,
        "retry_count": "-1",
        "url_domain": "192.168.189.156:12346"
    },
}


class TestApi(base.TestCase):

    def test_basic(self):
        sot = api.Api()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/apis',
            sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_fetch)
        self.assertEqual('apis', sot.resources_key)

    def test_make_it(self):
        sot = api.Api(**EXAMPLE)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['group_id'], sot.group_id)
        self.assertEqual(EXAMPLE['auth_type'], sot.auth_type)
        self.assertEqual(EXAMPLE['backend_type'], sot.backend_type)
        self.assertEqual(EXAMPLE['req_protocol'], sot.req_protocol)


EXAMPLE_PUB = {
    "api_id": "5f918d104dc84480a75166ba99efff21",
    "env_id": "DEFAULT_ENVIRONMENT_RELEASE_ID",
    "publish_id": "9191cdb430724d4b8586ed7f1b962ca2",
    "publish_time": "2020-08-03T01:36:00.592970615Z",
    "version_id": "ee1a5a38d3d3493abf1dc4ed6cacfa0b"
}


class TestPublishApi(base.TestCase):

    def test_basic(self):
        sot = _as.PublishApi()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/apis/action',
            sot.base_path)
        self.assertTrue(sot.allow_create)

    def test_make_it(self):
        sot = _as.PublishApi(**EXAMPLE_PUB)
        self.assertEqual(EXAMPLE_PUB['api_id'], sot.api_id)
        self.assertEqual(EXAMPLE_PUB['env_id'], sot.env_id)
        self.assertEqual(EXAMPLE_PUB['publish_id'], sot.publish_id)
        self.assertEqual(EXAMPLE_PUB['publish_time'], sot.publish_time)
        self.assertEqual(EXAMPLE_PUB['version_id'], sot.version_id)


EXAMPLE_CHECK = {
    "type": "name",
    "name": "api_demo"
}


class TestCheckApi(base.TestCase):

    def test_basic(self):
        sot = _as.CheckApi()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/apis/check',
            sot.base_path)
        self.assertTrue(sot.allow_create)

    def test_make_it(self):
        sot = _as.CheckApi(**EXAMPLE_CHECK)
        self.assertEqual(EXAMPLE_CHECK['type'], sot.type)
        self.assertEqual(EXAMPLE_CHECK['name'], sot.name)


EXAMPLE_DEBUG = {
    "request": "GET /test/mock HTTP/1.1\n"
               "Host: c77f5e81d9cb4424bf704ef2b0ac7600.apic.****.com\n"
               "User-Agent: APIGatewayDebugClient/1.0\n"
               "X-Apig-Mode: debug\n"
               "\n",
    "response": "HTTP/1.1 200 OK\n"
                "Transfer-Encoding: chunked\n"
                "Connection: keep-alive\n"
                "Content-Type: application/json\n"
                "Date: Mon, 03 Aug 2020 02:51:22 GMT\n"
                "Server: api-gateway\n"
                "X-Apig-Latency: 0\n"
                "X-Apig-Ratelimit-Api: remain:99,limit:100,time:1 minute\n"
                "X-Apig-Ratelimit-Api-Allenv: remain:14999,"
                "limit:15000,time:1 second\n"
                "X-Request-Id: d4ec6e33148bdeffe8f55b43472d1251\n"
                "\nmock success",
    "latency": 5,
    "log": ""
}


class TestDebugApi(base.TestCase):

    def test_basic(self):
        sot = _as.DebugApi()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/apis/debug/%(api_id)s',
            sot.base_path)
        self.assertTrue(sot.allow_create)

    def test_make_it(self):
        sot = _as.DebugApi(**EXAMPLE_DEBUG)
        self.assertEqual(EXAMPLE_DEBUG['request'], sot.request)
        self.assertEqual(EXAMPLE_DEBUG['response'], sot.response)
        self.assertEqual(EXAMPLE_DEBUG['latency'], sot.latency)
        self.assertEqual(EXAMPLE_DEBUG['log'], sot.log)


EXAMPLE_PUB_APIS = {
    "apis": [
        "3a955b791bd24b1c9cd94c745f8d1aad",
        "abd9c4b2ff974888b0ba79be7e6b2762"
    ],
    "env_id": "DEFAULT_ENVIRONMENT_RELEASE_ID",
    "group_id": "c77f5e81d9cb4424bf704ef2b0ac7600",
    "remark": "Published to the production environment"
}


class TestPublishApis(base.TestCase):

    def test_basic(self):
        sot = _as.PublishApis()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/apis/publish',
            sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):
        sot = _as.PublishApis(**EXAMPLE_PUB_APIS)
        self.assertEqual(EXAMPLE_PUB_APIS['apis'], sot.apis)
        self.assertEqual(EXAMPLE_PUB_APIS['env_id'], sot.env_id)
        self.assertEqual(EXAMPLE_PUB_APIS['group_id'], sot.group_id)
        self.assertEqual(EXAMPLE_PUB_APIS['remark'], sot.remark)


EXAMPLE_DEF = {
    "name": "Api_http",
    "type": 1,
    "version": "V0.0.1",
    "req_protocol": "HTTPS",
    "req_method": "GET",
    "req_uri": "/test/http",
    "auth_type": "AUTHORIZER",
    "authorizer_id": "8d0443832a194eaa84244e0c1c1912ac",
    "auth_opt": {
        "app_code_auth_type": "DISABLE"
    },
    "cors": False,
    "match_mode": "NORMAL",
    "backend_type": "HTTP",
    "remark": "Web backend API",
    "group_id": "c77f5e81d9cb4424bf704ef2b0ac7600",
    "result_normal_sample": "Example success response",
    "result_failure_sample": "Example failure response",
    "id": "5f918d104dc84480a75166ba99efff21",
    "group_name": "api_group_001",
    "run_env_id": "DEFAULT_ENVIRONMENT_RELEASE_ID",
    "run_env_name": "RELEASE",
    "publish_id": "9191cdb430724d4b8586ed7f1b962ca2",
    "sl_domain": "c77f5e81d9cb4424bf704ef2b0ac7600.apic.****.com",
    "sl_domains": [
        "c77f5e81d9cb4424bf704ef2b0ac7600.apic.****.com",
        "c77f5e81d9cb4424bf704ef2b0ac7600.apic.****.cn"
    ]
}


class TestRuntimeDefinition(base.TestCase):

    def test_basic(self):
        sot = _as.RuntimeDefinitionApi()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/apis/runtime/%(api_id)s',
            sot.base_path)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = _as.RuntimeDefinitionApi(**EXAMPLE_DEF)
        self.assertEqual(EXAMPLE_DEF['name'], sot.name)
        self.assertEqual(EXAMPLE_DEF['type'], sot.type)
        self.assertEqual(EXAMPLE_DEF['version'], sot.version)
        self.assertEqual(EXAMPLE_DEF['req_protocol'], sot.req_protocol)
        self.assertEqual(EXAMPLE_DEF['req_method'], sot.req_method)
        self.assertEqual(EXAMPLE_DEF['req_uri'], sot.req_uri)
        self.assertEqual(EXAMPLE_DEF['auth_type'], sot.auth_type)
        self.assertEqual(EXAMPLE_DEF['authorizer_id'], sot.authorizer_id)
        self.assertEqual(EXAMPLE_DEF['auth_opt'], sot.auth_opt)
        self.assertEqual(EXAMPLE_DEF['cors'], sot.cors)


EXAMPLE_VERSIONS = {
    "name": "Api_http",
    "type": 1,
    "version": "V0.0.1",
    "req_protocol": "HTTPS",
    "req_method": "GET",
    "req_uri": "/test/http",
    "auth_type": "AUTHORIZER",
}


class TestVersionsApi(base.TestCase):

    def test_basic(self):
        sot = _as.VersionsApi()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/apis/versions/%(version_id)s',
            sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = _as.VersionsApi(**EXAMPLE_VERSIONS)
        self.assertEqual(EXAMPLE_VERSIONS['name'], sot.name)
        self.assertEqual(EXAMPLE_VERSIONS['type'], sot.type)
        self.assertEqual(EXAMPLE_VERSIONS['version'], sot.version)
        self.assertEqual(EXAMPLE_VERSIONS['req_protocol'], sot.req_protocol)
        self.assertEqual(EXAMPLE_VERSIONS['req_method'], sot.req_method)
        self.assertEqual(EXAMPLE_VERSIONS['req_uri'], sot.req_uri)
        self.assertEqual(EXAMPLE_VERSIONS['auth_type'], sot.auth_type)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Connect to an OpenStack cloud.

For a full guide see TODO(etoews):link to docs on developer.openstack.org
"""

import argparse
import os

import openstack
from openstack.config import loader
import sys

openstack.enable_logging(True, stream=sys.stdout)

#: Defines the OpenStack Config loud key in your config file,
#: typically in $HOME/.config/openstack/clouds.yaml. That configuration
#: will determine where the examples will be run and what resource defaults
#: will be used to run the examples.
TEST_CLOUD = os.getenv('OS_TEST_CLOUD', 'devstack-admin')
config = loader.OpenStackConfig()
cloud = openstack.connect(cloud=TEST_CLOUD)


class Opts(object):
    def __init__(self, cloud_name='devstack-admin', debug=True):
        self.cloud = cloud_name
        self.debug = debug
        # Use identity v3 API for examples.
        self.identity_api_version = '3'


def _get_resource_value(resource_key, default):
    return config.get_extra_config('example').get(resource_key, default)


SERVER_NAME = 'openstacksdk-example'
IMAGE_NAME = _get_resource_value('image_name', 'cirros-0.3.5-x86_64-disk')
FLAVOR_NAME = _get_resource_value('flavor_name', 'm1.small')
NETWORK_NAME = _get_resource_value('network_name', 'private')
KEYPAIR_NAME = _get_resource_value('keypair_name', 'openstacksdk-example')
SSH_DIR = _get_resource_value(
    'ssh_dir', '{home}/.ssh'.format(home=os.path.expanduser("~")))
PRIVATE_KEYPAIR_FILE = _get_resource_value(
    'private_keypair_file', '{ssh_dir}/id_rsa.{key}'.format(
        ssh_dir=SSH_DIR, key=KEYPAIR_NAME))

EXAMPLE_IMAGE_NAME = 'openstacksdk-example-public-image'


def create_connection_from_config():
    return openstack.connect(cloud=TEST_CLOUD)


def create_connection_from_args():
    parser = argparse.ArgumentParser()
    config = loader.OpenStackConfig()
    config.register_argparse_arguments(parser, sys.argv[1:])
    args = parser.parse_args()
    return openstack.connect(config=config.get_one(argparse=args))


def create_connection(auth_url, region, project_name, username, password):

    return openstack.connect(
        auth_url=auth_url,
        project_name=project_name,
        username=username,
        password=password,
        region_name=region,
        app_name='examples',
        app_version='1.0',
    )



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Create Auto-Scaling Configuration.
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')

attrs = {
    'name': 'test-config',
    'instance_config': {
        'flavorRef': 's2.medium.1',
        'imageRef': '1616e0b6-503a-4698-946f-cf9942c4c73b',
        'disk': [{
            'size': 20,
            'volume_type': 'SATA',
            'disk_type': 'SYS'
        }],
        'key_name': 'test-key',
    }
}

config = conn.auto_scaling.create_config(**attrs)
print(config)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import mock

from otcextensions.sdk.cce.v3 import cluster
from otcextensions.sdk.cce.v3 import cluster_node as clusterNode
from otcextensions.osclient.cce.v2 import cluster_node
from otcextensions.tests.unit.osclient.cce.v2 import fakes


class TestCreateClusterNode(fakes.TestCCE):

    _obj = fakes.FakeClusterNode.create_one()

    columns = (
        'id',
        'name',
        'private_ip',
        'public_ip',
        'status',
        'flavor',
        'ssh_key',
        'availability_zone',
        'operating_system',
        'root_volume_type',
        'root_volume_size',
        'data_volume_type_1',
        'data_volume_size_1',
        'data_volume_type_2',
        'data_volume_size_2')

    flat_data = cluster_node._flatten_cluster_node(_obj)

    data = (
        flat_data['id'],
        flat_data['name'],
        flat_data['private_ip'],
        flat_data['public_ip'],
        flat_data['status'],
        flat_data['flavor'],
        flat_data['ssh_key'],
        flat_data['availability_zone'],
        flat_data['operating_system'],
        flat_data['root_volume_type'],
        flat_data['root_volume_size'],
        flat_data['data_volume_type_1'],
        flat_data['data_volume_size_1'],
        flat_data['data_volume_type_2'],
        flat_data['data_volume_size_2']
    )

    def setUp(self):
        super(TestCreateClusterNode, self).setUp()

        self.cmd = cluster_node.CreateCCEClusterNode(self.app, None)

        self.app.client_manager.sdk_connection = mock.Mock()

        self.cloud_client = self.app.client_manager.sdk_connection

        self.cloud_client.create_cce_cluster_node = mock.Mock()

        self.client.find_cluster = mock.Mock(
            return_value=cluster.Cluster(id='cluster_id'))

    def test_create(self):
        arglist = [
            'cluster_name',
            '--name', 'node_name',
            '--annotation', 'a1=v1',
            '--annotation', 'a2=v2',
            '--availability-zone', 'az',
            '--bandwidth', '30',
            '--count', '12',
            '--data-volume', 'volumetype=SATA,size=100',
            '--data-volume', 'volumetype=SAS,size=200',
            '--flavor', 'flvr',
            '--label', 'l1=v1',
            '--label', 'l2=v2',
            '--network', 'nw',
            '--root-volume-size', '230',
            '--root-volume-type', 'SATA',
            '--ssh-key', 'key',
        ]

        verifylist = [
            ('cluster', 'cluster_name'),
            ('name', 'node_name'),
            ('annotations', {'a1': 'v1', 'a2': 'v2'}),
            ('availability_zone', 'az'),
            ('bandwidth', 30),
            ('count', 12),
            ('data_volumes', [
                {'volumetype': 'SATA', 'size': '100'},
                {'volumetype': 'SAS', 'size': '200'}]),
            ('flavor', 'flvr'),
            ('labels', {'l1': 'v1', 'l2': 'v2'}),
            ('network', 'nw'),
            ('root_volume_size', 230),
            ('root_volume_type', 'SATA'),
            ('ssh_key', 'key'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.cloud_client.create_cce_cluster_node.side_effect = [
            self._obj
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.cloud_client.create_cce_cluster_node.assert_called_once_with(
            availability_zone='az',
            cluster='cluster_name',
            count=12,
            flavor='flvr',
            network='nw',
            ssh_key='key',
            annotations={'a1': 'v1', 'a2': 'v2'},
            bandwidth=30,
            data_volumes=[
                {'volumetype': 'SATA', 'size': '100'},
                {'volumetype': 'SAS', 'size': '200'}],
            labels={'l1': 'v1', 'l2': 'v2'},
            name='node_name',
            root_volume_size=230,
            root_volume_type='SATA',
            wait=False,
            wait_interval=5,
            wait_timeout=3600
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestClusterNode(fakes.TestCCE):

    def setUp(self):
        super(TestClusterNode, self).setUp()

    def test_flatten(self):
        _obj = fakes.FakeClusterNode.create_one()

        flat_data = cluster_node._flatten_cluster_node(_obj)

        data = (
            flat_data['id'],
            flat_data['name'],
            flat_data['flavor'],
            flat_data['private_ip'],
            flat_data['public_ip'],
            flat_data['availability_zone'],
            flat_data['ssh_key'],
            flat_data['status'],
        )

        cmp_data = (
            _obj.metadata.id,
            _obj.metadata.name,
            _obj.spec.flavor,
            _obj.status.private_ip,
            _obj.status.floating_ip,
            _obj.spec.availability_zone,
            _obj.spec.login.get('sshKey', None),
            _obj.status.status,
        )

        self.assertEqual(data, cmp_data)


class TestListClusterNode(fakes.TestCCE):

    _objs = fakes.FakeClusterNode.create_multiple(3)

    columns = ('ID', 'name', 'private_ip', 'public_ip',
               'status')

    data = []

    for s in _objs:
        flat_data = cluster_node._flatten_cluster_node(s)

        data.append((
            flat_data['id'],
            flat_data['name'],
            flat_data['private_ip'],
            flat_data['public_ip'],
            flat_data['status'],
        ))

    def setUp(self):
        super(TestListClusterNode, self).setUp()

        self.cmd = cluster_node.ListCCEClusterNode(self.app, None)

        self.client.cluster_nodes = mock.Mock()
        self.client.find_cluster = mock.Mock(
            return_value=cluster.Cluster(id='cluster_id'))

    def test_list_default(self):
        arglist = ['cluster_id']

        verifylist = [
            ('cluster', 'cluster_id')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.cluster_nodes.side_effect = [
            self._objs
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.cluster_nodes.assert_called_once_with(
            'cluster_id'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowClusterNode(fakes.TestCCE):

    _obj = fakes.FakeClusterNode.create_one()

    columns = (
        'id',
        'name',
        'private_ip',
        'public_ip',
        'status',
        'flavor',
        'ssh_key',
        'availability_zone',
        'operating_system',
        'root_volume_type',
        'root_volume_size',
        'data_volume_type_1',
        'data_volume_size_1',
        'data_volume_type_2',
        'data_volume_size_2')

    flat_data = cluster_node._flatten_cluster_node(_obj)

    data = (
        flat_data['id'],
        flat_data['name'],
        flat_data['private_ip'],
        flat_data['public_ip'],
        flat_data['status'],
        flat_data['flavor'],
        flat_data['ssh_key'],
        flat_data['availability_zone'],
        flat_data['operating_system'],
        flat_data['root_volume_type'],
        flat_data['root_volume_size'],
        flat_data['data_volume_type_1'],
        flat_data['data_volume_size_1'],
        flat_data['data_volume_type_2'],
        flat_data['data_volume_size_2']
    )

    def setUp(self):
        super(TestShowClusterNode, self).setUp()

        self.cmd = cluster_node.ShowCCEClusterNode(self.app, None)

        self.client.find_cluster_node = mock.Mock()

        self.client.find_cluster = mock.Mock(
            return_value=cluster.Cluster(id='cluster_uuid'))

    def test_show(self):
        arglist = [
            'cluster_uuid',
            'node_id'
        ]

        verifylist = [
            ('cluster', 'cluster_uuid'),
            ('node', 'node_id')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_cluster_node.side_effect = [
            self._obj
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_cluster_node.assert_called_once_with(
            cluster='cluster_uuid',
            node='node_id')

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteClusterNode(fakes.TestCCE):

    def setUp(self):
        super(TestDeleteClusterNode, self).setUp()

        self.cmd = cluster_node.DeleteCCEClusterNode(self.app, None)

        self.client.delete_cluster_node = mock.Mock()

        self.client.find_cluster = mock.Mock(
            return_value=cluster.Cluster(id='cluster_uuid'))

        self.client.find_cluster_node = mock.Mock()

    def test_delete(self):
        arglist = [
            'cluster_uuid',
            'node1',
            'node2'
        ]

        verifylist = [
            ('cluster', 'cluster_uuid'),
            ('node', ['node1', 'node2'])
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_cluster_node.side_effect = [{}, {}]

        # Set the response for find_cluster
        self.client.find_cluster_node.side_effect = [
            clusterNode.ClusterNode(id='node1'),
            clusterNode.ClusterNode(id='node2')]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        delete_calls = [
            mock.call(cluster='cluster_uuid', node='node1',
                      ignore_missing=False),
            mock.call(cluster='cluster_uuid', node='node2',
                      ignore_missing=False)
        ]

        find_calls = [
            mock.call(cluster='cluster_uuid', node='node1'),
            mock.call(cluster='cluster_uuid', node='node2')
        ]

        self.client.delete_cluster_node.assert_has_calls(delete_calls)
        self.client.find_cluster_node.assert_has_calls(find_calls)
        self.assertEqual(2, self.client.delete_cluster_node.call_count)



#   Copyright 2013 Nebula Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#


def parse_s3_uri(s3path):
    bucket, path = '', ''
    if str(s3path) and s3path.startswith('s3://'):
        val = str(s3path).replace('s3://', '').split('/', 1)
        bucket = val[0]
        if len(val) > 1:
            path = val[1].strip('/')
        else:
            path = ""
    return bucket, path



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base

from otcextensions.sdk.dds.v3 import flavor

EXAMPLE = {
    "region": "eu-de",
    "engine_name": "engine",
}


class TestFlavor(base.TestCase):

    def test_basic(self):
        sot = flavor.Flavor()

        self.assertEqual('/flavors', sot.base_path)

        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_commit)
        self.assertDictEqual({'limit': 'limit',
                              'marker': 'marker',
                              'region': 'region',
                              'engine_name': 'engine_name'},
                             sot._query_mapping._mapping)

    def test_make_it(self):
        sot = flavor.Flavor(**EXAMPLE)
        self.assertEqual(EXAMPLE['region'], sot.region)
        self.assertEqual(EXAMPLE['engine_name'], sot.engine_name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class FloatingIP(resource.Resource):

    resources_key = 'ddosStatus'
    base_path = '/antiddos'

    # capabilities
    allow_list = True
    allow_fetch = True
    allow_delete = True
    allow_commit = True

    _query_mapping = resource.QueryParameters('status',
                                              'limit',
                                              'offset',
                                              'ip')

    # Properties
    floating_ip_id = resource.Body('floating_ip_id', alternate_id=True)
    #: Floating ipaddress
    floating_ip_address = resource.Body('floating_ip_address')
    #: Flag to indicate whether L7 protection is enabled
    #: *Type: bool*
    is_enable_l7 = resource.Body('enable_L7', type=bool)
    #: Traffic segment id
    #: Valid values are from `1` to `9`
    #: *Type: int*
    traffic_pos_id = resource.Body('traffic_pos_id', type=int)
    #: Http request segment id
    #: Valid values are from `1` to `15`
    #: *Type: int*
    http_request_pos_id = resource.Body('http_request_pos_id', type=int)
    #: Cleaning access limit id
    #: Valid values are from `1` to `8`
    #: *Type: int*
    cleaning_access_pos_id = resource.Body('cleaning_access_pos_id', type=int)
    #: Application type id
    #: Valid values are from `0`, `1`
    #: *Type: int*
    app_type_id = resource.Body('app_type_id', type=int)

    #: Network type: EIP or ELB
    network_type = resource.Body('network_type')
    #: Status of the EIP
    #: Possible values:
    #: * `normal`: indicates that the defense status is normal.
    #: * `configging`: indicates that defense is being configured.
    #: * `notConfig`: indicates that defense is not configured.
    #: * `packetcleaning`: indicates that traffic cleaning is underway.
    #: * `packetdropping`: indicates that traffic is discarded.
    #: *Type: str*
    status = resource.Body('status')
    #: Task id
    task_id = resource.Body('task_id')

    def update(self, session, prepend_key=True, has_body=True):
        # floating update requires body to have `enable_L7, traffic_pos_id
        # http_request_pos_id, cleaning_access_pos_id, app_type_id`
        # so enforce them dirty

        self._body._dirty.add('enable_L7')
        self._body._dirty.add('traffic_pos_id')
        self._body._dirty.add('http_request_pos_id')
        self._body._dirty.add('cleaning_access_pos_id')
        self._body._dirty.add('app_type_id')

        return super(FloatingIP, self).update(
            session,
            prepend_key=prepend_key,
            has_body=has_body)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import test_proxy_base

from otcextensions.sdk.volume_backup.v2 import _proxy
from otcextensions.sdk.volume_backup.v2 import backup_policy as _backup_policy
from otcextensions.sdk.volume_backup.v2 import backup_task as _backup_task
from otcextensions.sdk.volume_backup.v2 import job as _job


class TestVolumeBackupProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestVolumeBackupProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestBackupPolicy(TestVolumeBackupProxy):

    def test_list(self):
        self.verify_list(
            self.proxy.backup_policies, _backup_policy.BackupPolicy,
            mock_method='openstack.proxy.Proxy._list',
            method_kwargs={
            },
            expected_kwargs={
                'paginated': False,
            }
        )

    def test_find(self):
        self.verify_find(
            self.proxy.find_backup_policy,
            _backup_policy.BackupPolicy,
            mock_method='openstack.proxy.Proxy._find',
        )

    def test_create(self):
        self.verify_create(
            self.proxy.create_backup_policy, _backup_policy.BackupPolicy,
            mock_method='openstack.proxy.Proxy._create',
            method_kwargs={
                'name': 'some_name'
            },
            expected_kwargs={
                'name': 'some_name'
            }
        )

    def test_delete(self):
        self.verify_delete(
            self.proxy.delete_backup_policy,
            _backup_policy.BackupPolicy, True,
            mock_method='openstack.proxy.Proxy._delete',
            expected_kwargs={
            }
        )

    def test_update(self):
        self._verify(
            'openstack.proxy.Proxy._update',
            self.proxy.update_backup_policy,
            method_args=['INSTANCE'],
            method_kwargs={'test': 't'},
            expected_args=[_backup_policy.BackupPolicy, 'INSTANCE'],
            expected_kwargs={
                'test': 't',
                'prepend_key': False,
            }
        )

    def test_execute_policy(self):
        self._verify(
            'otcextensions.sdk.volume_backup.v2.backup_policy.'
            'BackupPolicy.execute',
            self.proxy.execute_policy,
            method_args=['INSTANCE'],
            expected_args=[self.proxy],
        )

    def test_enable_policy(self):
        self._verify(
            'openstack.proxy.Proxy._update',
            self.proxy.enable_policy,
            method_args=['INSTANCE'],
            expected_args=[_backup_policy.BackupPolicy, 'INSTANCE'],
            expected_kwargs={
                'prepend_key': False,
                'scheduled_policy': {'status': 'ON'}
            }
        )

    def test_disable_policy(self):
        self._verify(
            'openstack.proxy.Proxy._update',
            self.proxy.disable_policy,
            method_args=['INSTANCE'],
            expected_args=[_backup_policy.BackupPolicy, 'INSTANCE'],
            expected_kwargs={
                'prepend_key': False,
                'scheduled_policy': {'status': 'OFF'}
            }
        )


class TestBackupPolicyResource(TestVolumeBackupProxy):
    def test_link(self):
        self._verify(
            'otcextensions.sdk.volume_backup.v2.backup_policy.'
            'BackupPolicyResource._process',
            self.proxy.link_resources_to_policy,
            method_args=['policy', ['r1', 'r2']],
            expected_args=[
                self.proxy,
                'policy',
                True,
                ['r1', 'r2']],
            expected_kwargs={
            }
        )

    def test_unlink(self):
        self._verify(
            'otcextensions.sdk.volume_backup.v2.backup_policy.'
            'BackupPolicyResource._process',
            self.proxy.unlink_resources_of_policy,
            method_args=['policy', ['r1', 'r2']],
            expected_args=[
                self.proxy,
                'policy',
                False,
                ['r1', 'r2']],
            expected_kwargs={
            }
        )


class TestTask(TestVolumeBackupProxy):

    def test_list(self):
        self.verify_list(
            self.proxy.tasks, _backup_task.BackupTask,
            mock_method='openstack.proxy.Proxy._list',
            method_args=['pol_id'],
            expected_kwargs={
                'policy_id': 'pol_id'
            },
            expected_args=[]
        )


class TestJob(TestVolumeBackupProxy):

    def test_get(self):
        self.verify_get(
            self.proxy.get_job,
            _job.Job,
            mock_method='openstack.proxy.Proxy._get',
            expected_kwargs={
            }
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class DomainName(resource.Resource):
    base_path = (f'/apigw/instances/%(gateway_id)s/'
                 f'api-groups/%(group_id)s/domains')

    allow_create = True
    allow_commit = True
    allow_delete = True
    allow_fetch = True

    gateway_id = resource.URI('gateway_id')
    group_id = resource.URI('group_id')

    # Minimum SSL version. TLS 1.1 and TLS 1.2 are supported.
    # Default: TLSv1.1
    min_ssl_version = resource.Body('min_ssl_version', type=str)
    # Whether to enable HTTP redirection to HTTPS.
    # The value false means disable and true means enable.
    # The default value is false.
    is_http_redirect_to_https = resource.Body(
        'is_http_redirect_to_https', type=bool
    )
    # Custom domain name.
    # It can contain a maximum of 255 characters and
    # must comply with domain name specifications.
    url_domain = resource.Body('url_domain', type=str)

    # Attributes
    # CNAME resolution status.
    # 1: not resolved
    # 2: resolving
    # 3: resolved
    # 4: resolution failed
    status = resource.Body('status', type=int)
    # Whether to enable client certificate verification.
    # This parameter is available only when a certificate is bound.
    # It is enabled by default if trusted_root_ca exists,
    # and disabled if trusted_root_ca does not exist.
    verified_client_certificate_enabled = resource.Body(
        'verified_client_certificate_enabled', type=bool
    )


class Certificate(resource.Resource):
    base_path = (f'/apigw/instances/%(gateway_id)s/'
                 f'api-groups/%(group_id)s/domains/'
                 f'%(domain_id)s/certificate')

    allow_create = True
    allow_delete = True
    allow_fetch = True

    gateway_id = resource.URI('gateway_id')
    group_id = resource.URI('group_id')
    domain_id = resource.URI('domain_id')

    # Certificate content.
    cert_content = resource.Body('cert_content', type=str)
    # Certificate name. It can contain 4 to 50 characters,
    # starting with a letter. Only letters, digits,
    # and underscores (_) are allowed.
    name = resource.Body('name', type=str)
    # Private key.
    private_key = resource.Body('private_key', type=str)

    # Attributes
    # Custom domain name.
    url_domain = resource.Body('url_domain', type=str)
    # CNAME resolution status.
    # 1: not resolved
    # 2: resolving
    # 3: resolved
    # 4: resolution failed
    status = resource.Body('status', type=int)
    # Minimum SSL version. TLS 1.1 and TLS 1.2 are supported.
    # Default: TLSv1.1
    min_ssl_version = resource.Body('min_ssl_version', type=str)
    # Whether to enable HTTP redirection to HTTPS.
    # The value false means disable and true means enable.
    # The default value is false.
    is_http_redirect_to_https = resource.Body(
        'is_http_redirect_to_https', type=bool
    )
    # Whether to enable client certificate verification.
    # This parameter is available only when a certificate is bound.
    # It is enabled by default if trusted_root_ca exists,
    # and disabled if trusted_root_ca does not exist.
    verified_client_certificate_enabled = resource.Body(
        'verified_client_certificate_enabled', type=bool
    )
    # Certificate name.
    ssl_name = resource.Body('ssl_name', type=str)
    # Certificate ID.
    ssl_id = resource.Body('ssl_id', type=str)
    # Certificate type. Options:
    # global: Global certificate.
    # instance: Gateway certificate.
    type = resource.Body('type', type=str)
    # Project ID.
    project_id = resource.Body('project_id', type=str)
    # Creation time.
    created_at = resource.Body('create_time', type=str)
    # Update time.
    updated_at = resource.Body('update_time', type=str)
    # Certificate domain name.
    common_name = resource.Body('common_name', type=str)
    # Subject alternative names.
    san = resource.Body('san', type=list)
    # Certificate version.
    version = resource.Body('version', type=int)
    # Company or organization.
    organization = resource.Body('organization', type=list)
    # Department
    organizational_unit = resource.Body('organizational_unit', type=list)
    # City
    locality = resource.Body('locality', type=list)
    # State or province.
    state = resource.Body('state', type=list)
    # Country or region.
    country = resource.Body('country', type=list)
    # Start time of the certificate validity period.
    not_before = resource.Body('not_before', type=str)
    # End time of the certificate validity period.
    not_after = resource.Body('not_after', type=str)
    # Serial No.
    serial_number = resource.Body('serial_number', type=str)
    # Certificate issuer.
    issuer = resource.Body('issuer', type=list)
    # Signature algorithm.
    signature_algorithm = resource.Body('signature_algorithm', type=str)


class DomainDebug(resource.Resource):
    base_path = (f'/apigw/instances/%(gateway_id)s/api-groups/'
                 f'%(group_id)s/sl-domain-access-settings')

    requires_id = False

    allow_commit = True

    gateway_id = resource.URI('gateway_id')
    group_id = resource.URI('group_id')

    # Specifies whether the debugging domain name is accessible.
    # Options: true and false.
    sl_domain_access_enabled = resource.Body(
        'sl_domain_access_enabled', type=bool
    )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from unittest.mock import call

import mock
from openstackclient.tests.unit import utils as tests_utils
from osc_lib import exceptions

from otcextensions.common import cli_utils
from otcextensions.osclient.modelartsv2.v2 import dataset
from otcextensions.tests.unit.osclient.modelartsv2.v2 import fakes

_COLUMNS = (
    "ai_project",
    "annotated_sample_count",
    "content_labeling",
    "create_time",
    "current_version_id",
    "current_version_name",
    "data_format",
    "data_sources",
    "data_update_time",
    "dataset_format",
    "dataset_id",
    "dataset_name",
    "dataset_type",
    "dataset_version",
    "dataset_version_count",
    "description",
    "enterprise_project_id",
    "feature_supports",
    "id",
    "import_data",
    "inner_annotation_path",
    "inner_data_path",
    "inner_log_path",
    "inner_task_path",
    "inner_temp_path",
    "inner_work_path",
    "label_task_count",
    "labels",
    "managed",
    "name",
    "next_version_num",
    "status",
    "total_sample_count",
    "unconfirmed_sample_count",
    "update_time",
    "versions",
    "work_path",
    "work_path_type",
    "workforce_task_count",
    "workspace_id",
)


class TestListDatasets(fakes.TestModelartsv2):
    objects = fakes.FakeDataset.create_multiple(3)

    column_list_headers = (
        "Dataset Id",
        "Dataset Name",
        "Dataset Type",
        "Status",
        "Total Sample Count",
        "Annotated Sample Count",
        "Create Time",
    )

    data = []

    for s in objects:
        data.append(
            (
                s.dataset_id,
                s.dataset_name,
                dataset.DatasetType(s.dataset_type),
                dataset.DatasetStatus(s.status),
                s.total_sample_count,
                s.annotated_sample_count,
                cli_utils.UnixTimestampFormatter(s.create_time),
            )
        )

    def setUp(self):
        super(TestListDatasets, self).setUp()

        self.cmd = dataset.ListDatasets(self.app, None)

        self.client.datasets = mock.Mock()
        self.client.api_mock = self.client.datasets

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            "--check-running-task",
            "--contain-versions",
            "--dataset-type",
            "0",
            "--file-preview",
            "--limit",
            "1",
            "--offset",
            "2",
            "--order",
            "asc",
            "--running-task-type",
            "0",
            "--search-content",
            "123",
            "--sort-by",
            "create_time",
            "--support-export",
            "--train-evaluate-ratio",
            "2,3",
            "--version-format",
            "0",
            "--with-labels",
            "--workspace-id",
            "0",
        ]

        verifylist = [
            ("check_running_task", True),
            ("contain_versions", True),
            ("dataset_type", 0),
            ("file_preview", True),
            ("limit", 1),
            ("offset", 2),
            ("order", "asc"),
            ("running_task_type", 0),
            ("search_content", "123"),
            ("sort_by", "create_time"),
            ("support_export", True),
            ("train_evaluate_ratio", "2,3"),
            ("version_format", 0),
            ("with_labels", True),
            ("workspace_id", "0"),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            check_running_task=True,
            contain_versions=True,
            dataset_type=0,
            file_preview=True,
            limit=1,
            offset=2,
            order="asc",
            running_task_type=0,
            search_content="123",
            sort_by="create_time",
            support_export=True,
            train_evaluate_ratio="2,3",
            version_format=0,
            with_labels=True,
            workspace_id="0",
        )


class TestShowDataset(fakes.TestModelartsv2):
    columns = _COLUMNS

    object = fakes.FakeDataset.create_one()

    data = fakes.gen_data(object, columns, dataset._formatters)

    def setUp(self):
        super(TestShowDataset, self).setUp()

        self.cmd = dataset.ShowDataset(self.app, None)

        self.client.find_dataset = mock.Mock(return_value=self.object)

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        # Testing that a call without the required argument will fail and
        # throw a "ParserExecption"
        self.assertRaises(
            tests_utils.ParserException,
            self.check_parser,
            self.cmd,
            arglist,
            verifylist,
        )

    def test_show(self):
        arglist = [
            "dataset-id",
        ]

        verifylist = [
            ("dataset", "dataset-id"),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_dataset.assert_called_with(
            "dataset-id", ignore_missing=False
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_show_non_existent(self):
        arglist = [
            "nonexisting-dataset-id",
        ]

        verifylist = [
            ("dataset", "nonexisting-dataset-id"),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        get_mock_result = exceptions.CommandError("Resource Not Found")
        self.client.find_dataset = mock.Mock(side_effect=get_mock_result)

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual("Resource Not Found", str(e))
        self.client.find_dataset.assert_called_with(
            "nonexisting-dataset-id", ignore_missing=False
        )


class TestDeleteDataset(fakes.TestModelartsv2):
    _data = fakes.FakeDataset.create_one()

    def setUp(self):
        super(TestDeleteDataset, self).setUp()

        self.client.find_dataset = mock.Mock(return_value=self._data)
        self.client.delete_dataset = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = dataset.DeleteDataset(self.app, None)

    def test_delete(self):
        arglist = [
            "dataset-name",
        ]

        verifylist = [
            ("dataset", ["dataset-name"]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_dataset.assert_called_with(
            "dataset-name", ignore_missing=False
        )
        self.client.delete_dataset.assert_called_with(self._data.id)
        self.assertIsNone(result)

    def test_multiple_delete(self):
        arglist = [
            "dataset1-id",
            "dataset2-id",
        ]

        verifylist = [
            ("dataset", ["dataset1-id", "dataset2-id"]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)

        delete_calls = [call(self._data.id), call(self._data.id)]
        self.client.delete_dataset.assert_has_calls(delete_calls)
        self.assertIsNone(result)

    def test_multiple_delete_with_exception(self):
        arglist = [
            "dataset-id",
            "nonexisting-dataset-id",
        ]

        verifylist = [
            ("dataset", ["dataset-id", "nonexisting-dataset-id"]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        delete_mock_results = [None, exceptions.CommandError]
        self.client.delete_dataset = mock.Mock(side_effect=delete_mock_results)

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual("1 of 2 Dataset(s) failed to delete.", str(e))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy

from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.rds.v1 import backup

RDS_HEADERS = {
    'content-type': 'application/json',
    'x-language': 'en-us'
}

# PROJECT_ID = '23'
IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'id': IDENTIFIER,
    'name': '50deafb3e45d451a9406ca146b71fe9a_rds',
    'description': '',
    'locationRef': '',
    'created': '2016-08-23T03:59:23',
    'updated': '2016-08-23T04:01:40',
    'size': 0.0,
    'status': 'COMPLETED',
    'backuptype': '1',
    'dataStore': {
        'type': 'MySQL',
        'version': '5.6.30',
        'version_id': 'e8a8b8cc-63f8-4fb5-8d4a-24c502317a61'
    },
    'instance_id': '4f87d3c4-9e33-482f-b962-e23b30d1a18c',
    'parent_id': None,
}

EXAMPLE_POLICY = {
    'keepday': 7,
    'starttime': '00:00:00'
}


class TestBackup(base.TestCase):

    def setUp(self):
        super(TestBackup, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.post = mock.Mock()
        self.sess.delete = mock.Mock()
        self.sess.put = mock.Mock()
        # self.sess.get_project_id = mock.Mock(return_value=PROJECT_ID)
        self.sot = backup.Backup(**EXAMPLE)

    def test_basic(self):
        sot = backup.Backup()
        self.assertEqual('backup', sot.resource_key)
        self.assertEqual('backups', sot.resources_key)
        self.assertEqual('/backups', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = backup.Backup(**EXAMPLE)
        self.assertEqual(IDENTIFIER, sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['created'], sot.created)

    def test_list(self):

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {'backups': [EXAMPLE]}

        self.sess.get.return_value = mock_response

        result = list(self.sot.list(
            self.sess,
            headers=RDS_HEADERS))

        self.sess.get.assert_called_once_with(
            '/backups',
            headers=RDS_HEADERS,
            params={},
        )

        self.assertEqual([backup.Backup(**EXAMPLE)], result)

    def test_create(self):

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {'backup': copy.deepcopy(EXAMPLE)}
        mock_response.headers = {}

        self.sess.post.return_value = mock_response

        sot = backup.Backup.new(
            # project_id=PROJECT_ID,
            name='backup_name',
            description='descr',
            instance='some_instance')

        result = sot.create(self.sess, headers=RDS_HEADERS)

        self.sess.post.assert_called_once_with(
            '/backups',
            headers=RDS_HEADERS,
            json={'backup': {
                'instance': 'some_instance',
                'description': 'descr',
                'name': 'backup_name'}
            }
        )

        self.assertEqual(
            backup.Backup(
                # project_id=PROJECT_ID,
                instance='some_instance',
                **EXAMPLE),
            result)

    def test_delete(self):

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {}
        mock_response.headers = {}

        self.sess.delete.return_value = mock_response

        sot = backup.Backup(
            # project_id=PROJECT_ID,
            **EXAMPLE
        )

        sot.delete(self.sess, headers=RDS_HEADERS)

        url = 'backups/%(id)s' % \
            {
                'id': sot.id
            }

        # utils.urljoin strips leading '/', but it is not a problem
        self.sess.delete.assert_called_once_with(
            url,
            headers=RDS_HEADERS
        )

    def test_policy_basic(self):
        sot = backup.BackupPolicy()
        self.assertEqual('policy', sot.resource_key)
        self.assertEqual(None, sot.resources_key)
        self.assertEqual('/instances/%(instance_id)s/'
                         'backups/policy', sot.base_path)
        self.assertFalse(sot.allow_list)
        self.assertFalse(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_update)
        self.assertFalse(sot.allow_delete)

    def test_policy_make_it(self):
        sot = backup.BackupPolicy(**EXAMPLE_POLICY)
        self.assertEqual(EXAMPLE_POLICY['keepday'], sot.keepday)
        self.assertEqual(EXAMPLE_POLICY['starttime'], sot.starttime)

    def test_policy_update(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {
            'policy':
            copy.deepcopy(EXAMPLE_POLICY)}
        mock_response.headers = {}
        instance_id = 'instance_id'

        self.sess.put.return_value = mock_response

        sot = backup.BackupPolicy.new(
            # project_id=PROJECT_ID,
            instance_id=instance_id,
            **EXAMPLE_POLICY)

        self.assertIsNone(sot.update(self.sess, headers=RDS_HEADERS))

        url = '/instances/%(instance_id)s/backups/policy' % \
            {
                'instance_id': instance_id
            }

        self.sess.put.assert_called_once_with(
            url,
            headers=RDS_HEADERS,
            json={'policy': EXAMPLE_POLICY}
        )

    def test_policy_get(self):

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {
            'policy':
            copy.deepcopy(EXAMPLE_POLICY)}
        mock_response.headers = {}
        instance_id = 'instance_id'

        self.sess.get.return_value = mock_response

        sot = backup.BackupPolicy.new(
            # **EXAMPLE_POLICY,
            # project_id=PROJECT_ID,
            instance_id=instance_id)

        res = sot.get(self.sess, requires_id=False, headers=RDS_HEADERS)

        url = '/instances/%(instance_id)s/backups/policy' % \
            {
                'instance_id': instance_id
            }

        self.sess.get.assert_called_once_with(
            url,
            headers=RDS_HEADERS
        )

        self.assertEqual(EXAMPLE_POLICY['keepday'], res.keepday)
        self.assertEqual(EXAMPLE_POLICY['starttime'], res.starttime)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import datetime

from openstack import proxy
from openstack import resource

from otcextensions.sdk import job
from otcextensions.sdk.rds.v3 import backup as _backup
from otcextensions.sdk.rds.v3 import configuration as _configuration
from otcextensions.sdk.rds.v3 import datastore as _datastore
from otcextensions.sdk.rds.v3 import flavor as _flavor
from otcextensions.sdk.rds.v3 import instance as _instance
from otcextensions.sdk.rds.v3 import storage_type as _storage_type


class Proxy(proxy.Proxy, job.JobProxyMixin):

    skip_discovery = True

    def __init__(self, session, *args, **kwargs):
        super(Proxy, self).__init__(session=session, *args, **kwargs)
        self.additional_headers = {
            'Content-Type': 'application/json',
            'X-Language': 'en-us'
        }

    # ======= Datastores =======
    def datastore_types(self):
        """List supported datastore types

        :returns: A generator of supported datastore type objects
            with name attribute.
        """
        for ds in ['MySQL', 'PostgreSQL', 'SQLServer']:
            obj = type('obj', (object, ), {'name': ds})
            yield obj

    def datastores(self, database_name):
        """List datastores

        :param database_name: database store name
            (MySQL, PostgreSQL, or SQLServer and is case-sensitive.)

        :returns: A generator of supported datastore versions.

        :rtype: :class:`~otcextensions.sdk.rds.v3.datastore.Datastore`
        """
        return self._list(
            _datastore.Datastore,
            database_name=database_name,
        )

    # ======= Flavors =======

    def flavors(self, datastore_name, **params):
        """List flavors of given datastore_name and other optional parameters

        :param datastore_name: datastore_name
        :param version_name: version_name
        :param spec_code: spec_code

        :returns: A generator of flavor objects.
        """
        return self._list(_flavor.Flavor,
                          datastore_name=datastore_name,
                          **params)

    # ======= Storage Types =======

    def storage_types(self, datastore_name, version_name):
        """List the storage types of a specified DB engine version

        :param datastore_name: datastore_name
        :param version_name: version_name

        :returns: A generator of flavor objects.
        """
        return self._list(_storage_type.StorageType,
                          datastore_name=datastore_name,
                          version_name=version_name)

    # ======= Instance =======

    def create_instance(self, **attrs):
        """Create a new instance from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.rds.v3.instance.Instance`,
            comprised of the properties on the Instance class.

        :returns: The result of an instance creation.

        :rtype: :class:`~otcextensions.sdk.rds.v3.instance.Instance`
        """
        return self._create(_instance.Instance, **attrs)

    def delete_instance(self, instance, ignore_missing=True):
        """Delete an instance

        :param instance: The value can be either the ID of an instance or a
            :class:`~otcextensions.sdk.rds.v3.instance.Instance` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the instance does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent instance.

        :returns: ``None``
        """
        return self._delete(
            _instance.Instance,
            instance,
            ignore_missing=ignore_missing,
        )

    def get_instance(self, instance):
        """Get a single instance

        :param instance: The value can be either the ID of an instance or a
            :class:`~otcextensions.sdk.rds.v3.instance.Instance` instance.

        :returns: One :class:`~otcextensions.sdk.rds.v3.instance.Instance`
        """
        return self._get(_instance.Instance, instance)

    def find_instance(self, name_or_id, ignore_missing=True):
        """Find a single instance

        :param name_or_id: The name or ID of a instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.

        :returns:
            One :class:`~otcextensions.sdk.rds.v3.instance.Instance` or None.
        """
        return self._find(_instance.Instance,
                          name_or_id,
                          ignore_missing=ignore_missing)

    def instances(self, **params):
        """Return a generator of instances

        :param dict params: Optional query parameters to be sent to limit
            the instances being returned.

        :returns: A generator of instance objects.
        """
        return self._list(_instance.Instance, **params)

    def restore_instance(self,
                         instance,
                         backup=None,
                         restore_time=None):
        """Restore instance from backup or Restore using Point in
            Time Recovery

        :param instance: Either the id of a source target instance or a
            :class:`~otcextensions.sdk.rds.v3.instance.Instance` instance.
        :param backup: Either the id of a backup or a
            :class:`~otcextensions.sdk.rds.v3.backup.Backup` instance.

        :returns: Job ID
        """
        instance = self._get_resource(_instance.Instance, instance)
        if backup:
            backup = self._get_resource(_backup.Backup, backup)
        return instance.restore(self, backup, restore_time)

    def get_instance_restore_time(self, instance):
        """Obtaining a restore time of an instance.

        :param instance: This parameter can be either the ID of an instance
            or a :class:`~openstack.sdk.rds.v3.instance.Instance`

        :returns: Instance restore time

        :rtype: list

        """
        instance = self._get_resource(_instance.Instance, instance)
        return instance.fetch_restore_times(self)

    def get_instance_backup_policy(self, instance):
        """Obtaining a backup policy of the instance

        :param instance: This parameter can be either the ID of an instance
            or a :class:`~openstack.sdk.rds.v3.instance.Instance`

        :returns: Instance Backup policy.

        :rtype: :class:`~otcextensions.sdk.rds.v3.instance.BackupPolicy`
        """
        instance = self._get_resource(_instance.Instance, instance)
        return instance.get_backup_policy(self)

    def set_instance_backup_policy(self, instance, keep_days,
                                   start_time=None, period=None):
        """Sets the backup policy of the instance

        :param instance: This parameter can be either the ID of an instance
            or a :class:`~openstack.sdk.rds.v3.instance.Instance`
        :param keep_days:
        :param start_time:
        :param period:

        :returns: ``None``
        """
        instance = self._get_resource(_instance.Instance, instance)
        return instance.set_backup_policy(self, keep_days=keep_days,
                                          start_time=start_time, period=period)

    def restart_instance(self, instance):
        """Restart the database instance

        :param instance: This parameter can be either the ID of an instance
            or a :class:`~openstack.sdk.rds.v3.instance.Instance`
        """
        instance = self._get_resource(_instance.Instance, instance)
        return instance.restart(self)

    def enlarge_instance_volume(self, instance, size):
        """Enlarge the instance volume

        :param instance: This parameter can be either the ID of an instance
            or a :class:`~openstack.sdk.rds.v3.instance.Instance`
        :param size:
        """
        instance = self._get_resource(_instance.Instance, instance)
        return instance.enlarge_volume(self, size)

    def change_instance_flavor(self, instance, spec_code):
        """Change the instance's flavor.

        :param instance: This parameter can be either the ID of an instance
            or a :class:`~openstack.sdk.rds.v3.instance.Instance`
        :param spec_code:
        """
        instance = self._get_resource(_instance.Instance, instance)
        return instance.update_flavor(self, spec_code)

    def get_instance_logs(self, instance, log_type, start_date=None,
                          end_date=None, offset=1, limit=10, level='ALL'):
        """Get instance logs. If no dates are specified logs are gathered
            from the last 24 hours.

        :param instance: This parameter can be either the ID of an instance
            or a :class:`~openstack.sdk.rds.v3.instance.Instance`
        :param str log_type: The type of logs to query:
            'errorlog' or 'slowlog'.
        :param str start_date: Start date of the of the log query. Format:
            %Y-%m-%dT%H:%M:%S%z where z is the tzinfo in HHMM format.
        :param str end_date: End date of the of the log query. Format:
            %Y-%m-%dT%H:%M:%S%z where z is the tzinfo in HHMM format.
        :param int offset: Specifies the page offset such as 1, 2, 3 or 4.
        :param int limit: Specifies the number of records on a page.
            Its value range is from 1 to 100.
        :param str level: Specifies the log level.
            Values: ALL, INFO, LOG, WARNING, ERROR, FATAL, PANIC, NOTE.
        """
        instance = self._get_resource(_instance.Instance, instance)
        if log_type not in ['errorlog', 'slowlog']:
            raise Exception('The parameter log_type has to be either '
                            '"errorlog" or "slowlog".')
        if bool(start_date) ^ bool(end_date):
            raise Exception('The parameters start_date and end_date should '
                            'only be specified together.')
        elif not any([start_date, end_date]):
            current_time = datetime.datetime.now().astimezone()
            yesterday = current_time - datetime.timedelta(days=1)
            start_date = yesterday.strftime("%Y-%m-%dT%H:%M:%S%z")
            end_date = current_time.strftime("%Y-%m-%dT%H:%M:%S%z")

        return instance.get_logs(self, log_type, start_date, end_date,
                                 offset, limit, level)

    def add_tag(self, instance, key, value):
        """Add tag to instance.

        :param instance: This parameter can be either the ID of an instance
            or a :class:`~openstack.sdk.rds.v3.instance.Instance`
        :param str key: Name of the tag to be added.
        :param str value: Value of the tag to be added.
        """
        instance = self._get_resource(_instance.Instance, instance)
        return instance.add_tag(self, key, value)

    def remove_tag(self, instance, key):
        """Remove tag from instance.

        :param instance: This parameter can be either the ID of an instance
            or a :class:`~openstack.sdk.rds.v3.instance.Instance`
        :param str key: Name of the tag to be removed.
        """
        instance = self._get_resource(_instance.Instance, instance)
        return instance.remove_tag(self, key)

#     def get_instance_configuration(self, instance):
#         """Obtaining a Configuration associated to instance.
#
#         :param instance: This parameter can be either the ID of an instance
#             or a :class:`~openstack.sdk.rds.v3.instance.Instance`
#         :returns: Configuration Group details associated to instance
#         :rtype: :class:
#             `~otcextensions.sdk.rds.v3.instance.InstanceConfiguration`
#
#         """
#         instance = self._get_resource(instance)
#         return self._get(_instance.InstanceConfiguration,
#                          requires_id=False,
#                          instance_id=instance.id)
#
#     def update_instance_configuration(self, instance, **attrs):
#         """Updates the configuration params of the instance.
#
#         :param instance: This parameter can be either the ID of an instance
#             or a :class:`~openstack.sdk.rds.v3.instance.Instance`
#
#         :returns: Parameter restart required as bool value
#         """
#         instance = self._get_resource(_instance.Instance, instance)
#         return self._update(_instance.InstanceConfiguration,
#                             instance_id=instance.id,
#                             **attrs)
#
    # ======= Configurations =======
    def configurations(self, **params):
        """Obtaining a list of DB Configuration.

        :param dict params: Optional query parameters to be sent to limit
            the configurations being returned.

        :returns: A generator of Configuration objects.
        """
        return self._list(
            _configuration.Configuration,
            paginated=False
        )

    def get_configuration(self, cg):
        """Obtaining a Configuration.

        :param parameter_group: The value can be the ID of a Configuration or
            a :class:`~otcextensions.sdk.rds.v3.configuration.Configuration`
            instance.

        :returns:
            One :class:`~otcextensions.rds.v3.configuration.Configuration`
        """
        return self._get(_configuration.Configuration, cg)

    def find_configuration(self, name_or_id, ignore_missing=True):
        """Find Configuration by name or id.

        :param name_or_id: The name or ID of configuration.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.

        :returns: A configuration object.

        :rtype: :class:`~otcextensions.sdk.rds.v3.configuration.Configuration`
        """
        return self._find(_configuration.Configuration,
                          name_or_id,
                          ignore_missing=ignore_missing)

    def create_configuration(self, **attrs):
        """Create DB Configuration.

        :param dict attrs: Dict to overwrite Configuration object

        :returns: A Configuration object.

        :rtype:
            :class:`~otcextensions.sdk.rds.v3.configuration.Configuration`
        """
        return self._create(_configuration.Configuration,
                            prepend_key=False,
                            **attrs)

    def delete_configuration(self, cg, ignore_missing=True):
        """Delete DB Configuration.

        :param cg: The value can be the ID of a Configuration or a
            object of
            :class:`~otcextensions.sdk.rds.v3.configuration.Configuration`.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the Configration does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent Configuration.

        :returns: ``None``
        """
        self._delete(
            _configuration.Configuration,
            cg,
            ignore_missing=ignore_missing,
        )

    def update_configuration(self, config, **attrs):
        """Sets the configuration properties

        :param config: This parameter can be either the ID of a configuration
            or a :class:`~openstack.sdk.rds.v3.config.Configuration`
        :param dict attrs: The attributes to update on the configuration.

        :returns: ``None``
        """
        return self._update(_configuration.Configuration, config,
                            **attrs)

    def apply_configuration(self, configuration, instances):
        """Apply configuration to instances.

        :param configuration: The value can be the ID of a Configuration
            or a object of
            :class:`~otcextensions.sdk.rds.v3.configuration.Configuration`.
        :param instances: List of instance ids the configuration should be
            applied to

        :returns: Updated Configuration object.

        :rtype:
            :class:`~otcextensions.rds.v3.configuration.Configuration`.
        """
        cg = self._get_resource(_configuration.Configuration,
                                configuration)
        return cg.apply(self, instances)

    # ======= Backups =======
    def backups(self, instance, **params):
        """List Backups.

        :param instance: This parameter can be either the ID of an instance
            or a :class:`~openstack.sdk.rds.v3.instance.Instance`
        :param dict params: Optional query parameters to be sent to limit
            the instance backups being returned.

        :returns: A generator of backup objects.
        """
        instance = self._get_resource(_instance.Instance, instance)
        return self._list(_backup.Backup, instance_id=instance.id, **params)

    def create_backup(self, instance, **attrs):
        """Create a backups of instance

        :param instance: This parameter can be either the ID of an instance
            or a :class:`~openstack.sdk.rds.v3.instance.Instance`

        :returns: A new backup object.

        :rtype: :class:`~otcextensions.sdk.rds.v3.backup.Backup`
        """
        instance = self._get_resource(_instance.Instance, instance)
        attrs.update({'instance_id': instance.id})
        return self._create(_backup.Backup, **attrs)

    def delete_backup(self, backup, ignore_missing=True):
        """Deletes given backup

        :param instance: The value can be either the ID of an instance or a
            :class:`~otcextension.sdk.rds.v3.instance.Instance` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the instance does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent instance.

        :returns: ``None``
        """
        return self._delete(_backup.Backup,
                            backup,
                            ignore_missing=ignore_missing)

    def find_backup(self, name_or_id, instance, ignore_missing=True):
        """Find a single backup

        :param name_or_id: The name or ID of a instance.
        :param instance: The value can be either the ID of an instance or a
            :class:`~otcextension.sdk.rds.v3.instance.Instance` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.

        :returns: One :class:`~otcextensions.sdk.rds.v3.backup.Backup`
            or None
        """
        instance = self._get_resource(_instance.Instance, instance)
        return self._find(_backup.Backup,
                          name_or_id,
                          ignore_missing=ignore_missing,
                          instance_id=instance.id)

    def backup_download_links(self, backup_id):
        """Obtaining a backup file download links.

        :param backup_id:

        :returns: backup files download link

        :rtype: :class:`~otcextensions.sdk.rds.v3.backup.BackupFile`
        """
        return self._list(_backup.BackupFile, backup_id=backup_id)

    def wait_for_backup(self, backup, status='COMPLETED', failures=None,
                        interval=2, wait=300, attribute='status'):
        failures = ['FAILED'] if failures is None else failures
        return resource.wait_for_status(
            self, backup, status, failures, interval, wait)

    def wait_for_delete(self, res, interval=2, wait=120, callback=None):
        """Wait for a resource to be deleted.

        :param res: The resource to wait on to be deleted.
        :type resource: A :class:`~openstack.resource.Resource` object.
        :param interval: Number of seconds to wait before to consecutive
            checks. Default to 2.
        :param wait: Maximum number of seconds to wait before the change.
            Default to 120.
        :param callback: A callback function. This will be called with a single
            value, progress, which is a percentage value from 0-100.

        :returns: The resource is returned on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
            to delete failed to occur in the specified seconds.
        """
        return resource.wait_for_delete(self, res, interval, wait, callback)

    def _get_cleanup_dependencies(self):
        return {
            'rds': {
                'before': ['network']
            }
        }

    def _service_cleanup(
            self,
            dry_run=True,
            client_status_queue=None,
            identified_resources=None,
            filters=None,
            resource_evaluation_fn=None,
            skip_resources=None,
    ):
        if self.should_skip_resource_cleanup("instance", skip_resources):
            return

        instances = []
        for instance in self.instances():
            if not dry_run:
                if instance.tags:
                    for tag in instance.tags:
                        self.remove_tag(instance, tag['key'])
                        self._service_cleanup_del_res(
                            self.remove_tag,
                            tag['key'],
                            dry_run=dry_run,
                            client_status_queue=client_status_queue,
                            identified_resources=identified_resources,
                            filters=filters,
                            resource_evaluation_fn=resource_evaluation_fn,
                        )
            for backup in self.backups(instance):
                if 'Automated' not in backup.type:
                    self._service_cleanup_del_res(
                        self.delete_backup,
                        backup,
                        dry_run=dry_run,
                        client_status_queue=client_status_queue,
                        identified_resources=identified_resources,
                        filters=filters,
                        resource_evaluation_fn=resource_evaluation_fn,
                    )
            need_delete = self._service_cleanup_del_res(
                self.delete_instance,
                instance,
                dry_run=dry_run,
                client_status_queue=client_status_queue,
                identified_resources=identified_resources,
                filters=filters,
                resource_evaluation_fn=resource_evaluation_fn,
            )
            if not dry_run and need_delete:
                instances.append(instance)

        for instance in instances:
            self.wait_for_delete(instance)

        for config in self.configurations():
            if 'Default-' not in config.name:
                self._service_cleanup_del_res(
                    self.delete_configuration,
                    config,
                    dry_run=dry_run,
                    client_status_queue=client_status_queue,
                    identified_resources=identified_resources,
                    filters=filters,
                    resource_evaluation_fn=resource_evaluation_fn,
                )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""ModelArts model v1 action implementations"""
import logging

from osc_lib import exceptions
from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import cli_utils
from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


_formatters = {
    "input_params": cli_utils.YamlFormat,
    "output_params": cli_utils.YamlFormat,
    "dependencies": cli_utils.YamlFormat,
    "model_metrics": cli_utils.YamlFormat,
    "specification": cli_utils.YamlFormat,
    "apis": cli_utils.YamlFormat,
    "config": cli_utils.YamlFormat,
    "install_type": cli_utils.YamlFormat,
    "created_at": cli_utils.UnixTimestampFormatter,
}


def _get_columns(item):
    column_map = {}
    hidden = ["location"]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden
    )


def translate_response(func):
    def new(self, *args, **kwargs):
        obj = func(self, *args, **kwargs)
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)
        return (display_columns, data)

    new.__name__ = func.__name__
    new.__doc__ = func.__doc__
    return new


class DeleteModel(command.Command):
    _description = _("Delete ModelArts Model(s)")

    def get_parser(self, prog_name):
        parser = super(DeleteModel, self).get_parser(prog_name)
        parser.add_argument(
            "model",
            metavar="<model>",
            nargs="+",
            help=_("ID or Name of the ModelArts model(s) to be deleted."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1
        result = 0
        for name_or_id in parsed_args.model:
            try:
                model = client.find_model(name_or_id, ignore_missing=False)
                client.delete_model(model.id)
            except Exception as e:
                result += 1
                LOG.error(
                    _(
                        "Failed to delete model(s) with "
                        "ID or Name '%(model)s': %(e)s"
                    ),
                    {"model": name_or_id, "e": e},
                )
        if result > 0:
            total = len(parsed_args.model)
            msg = _(
                "%(result)s of %(total)s Model(s) failed " "to delete."
            ) % {"result": result, "total": total}
            raise exceptions.CommandError(msg)


class CreateModel(command.ShowOne):
    _description = _("Create a ModelArts model")

    def get_parser(self, prog_name):
        parser = super(CreateModel, self).get_parser(prog_name)
        parser.add_argument(
            "name",
            metavar="<name>",
            help=_(
                "Model name. The value can contain 1 to 64 visible "
                "characters, including Chinese characters. Only letters, "
                "Chinese characters, digits, hyphens (-), and "
                "underscores (_) are allowed."
            ),
        )
        parser.add_argument(
            "--model-version",
            metavar="<model_version>",
            required=True,
            help=_(
                "Model version in the format of Digit.Digit.Digit. "
                "The value range of the digits is [1, 99]. "
                "Note that no part of the version number can start with 0. "
                "For example, 01.01.01 is not allowed."
            ),
        )
        parser.add_argument(
            "--source-location",
            metavar="<source_location>",
            required=True,
            help=_(
                "OBS path where the model is located or the template "
                "address of the SWR image."
            ),
        )
        parser.add_argument(
            "--source-job-id",
            metavar="<source_job_id>",
            help=_(
                "ID of the source training job. If the model is generated "
                "from a training job, input this parameter for source "
                "tracing. If the model is imported from a third-party meta "
                "model, leave this parameter blank. "
                "By default, this parameter is left blank. "
            ),
        )
        parser.add_argument(
            "--source-job-version",
            metavar="<source_job_version>",
            help=_(
                "Version of the source training job. If the model is "
                "generated from a training job, input this parameter "
                "for source tracing. If the model is imported from a "
                "third-party meta model, leave this parameter blank. "
                "By default, this parameter is left blank."
            ),
        )
        parser.add_argument(
            "--source-type",
            metavar="<source_type>",
            help=_(
                "Model source type. Currently, the value can only be "
                "auto, which indicates ExeML models (model download is "
                "not supported). If the model is deployed by a training "
                "job, leave this parameter blank. By default, this "
                "parameter is left blank."
            ),
        )
        parser.add_argument(
            "--model-type",
            metavar="<model_type>",
            required=True,
            help=_(
                "Model type. The value can be TensorFlow, MXNet, Caffe, "
                "Spark_MLlib, Scikit_Learn, XGBoost, Image, or PyTorch, "
                "which is read from the configuration file."
            ),
        )
        parser.add_argument(
            "--runtime",
            metavar="<runtime>",
            help=_(
                "Model running environment. The possible values of "
                "runtime are related to model_type."
            ),
        )
        parser.add_argument(
            "--description",
            metavar="<description>",
            help=_(
                "Model remarks. The value contains a maximum of 100 "
                "characters and cannot contain the following special "
                "characters and more: &!'\"<>= "
            ),
        )
        parser.add_argument(
            "--execution-code",
            metavar="<execution_code>",
            help=_(
                "OBS path for storing the execution code. By default, "
                "this parameter is left blank. The name of the execution "
                "code file is fixed to customize_service.py. "
            ),
        )
        parser.add_argument(
            "--input-params",
            metavar="<input_params>",
            help=_(
                "Collection of input parameters of a model. By default, "
                "this parameter is left blank."
            ),
        )
        parser.add_argument(
            "--output-params",
            metavar="<output_params>",
            help=_(
                "Collection of output parameters of a model. By default, "
                "this parameter is left blank."
            ),
        )
        parser.add_argument(
            "--dependencies",
            metavar="<dependencies>",
            help=_(
                "Package required for inference code and model. "
                "By default, this parameter is left blank."
            ),
        )
        parser.add_argument(
            "--model-algorithm",
            metavar="<model_algorithm>",
            help=_(
                "Model algorithm. If the algorithm is read from the "
                "configuration file, this parameter can be left blank. "
                "For example, the value can be predict_analysis, "
                "object_detection, or image_classification."
            ),
        )
        parser.add_argument(
            "--model-metrics",
            metavar="<model_metrics>",
            help=_(
                "Model precision, which is read from the "
                "configuration file."
            ),
        )
        parser.add_argument(
            "--apis",
            metavar="<apis>",
            help=_(
                "All apis input and output parameters of the model. "
                "If the parameters are read from the configuration file, "
                "this parameter can be left blank."
            ),
        )
        parser.add_argument(
            "--initial-config",
            metavar="<initial_config>",
            help=_(
                "Character string converted from the final model "
                "configuration file. It is recommended that the "
                "initial_config file be used to provide information "
                "about the fields such as apis, dependencies, "
                "input_params, and output_params."
            ),
        )
        parser.add_argument(
            "--workspace-id",
            metavar="<workspace_id>",
            help=_("Workspace ID. Default value: 0"),
        )
        parser.add_argument(
            "--model-docs",
            metavar="<model_docs>",
            help=_(
                "List of model description documents. A maximum of three "
                "documents are supported."
            ),
        )
        parser.add_argument(
            "--install-type",
            metavar="<install_type>",
            help=_(
                "Deployment type. Only lowercase letters are supported. "
                "The value can be real-time, or batch. "
                'Default value: ["real-time","batch"]'
            ),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1
        attrs = {"model_name": parsed_args.name}

        args_list = (
            "model_version",
            "source_location",
            "source_job_id",
            "source_job_version",
            "source_type",
            "model_type",
            "runtime",
            "description",
            "execution_code",
            "input_params",
            "output_params",
            "dependencies",
            "model_algorithm",
            "model_metrics",
            "apis",
            "initial_config",
            "workspace_id",
            "model_docs",
            "install_type",
        )
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        data = client.create_model(**attrs)
        display_columns, columns = _get_columns(data)
        data = utils.get_item_properties(data, columns, formatters=_formatters)

        return display_columns, data


class ShowModel(command.ShowOne):
    _description = _("Show details of a modelartsv1 model")

    def get_parser(self, prog_name):
        parser = super(ShowModel, self).get_parser(prog_name)
        parser.add_argument(
            "model", metavar="<model>", help=_("Enter model id or name")
        )
        return parser

    @translate_response
    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1
        data = client.find_model(parsed_args.model, ignore_missing=False)
        if not data.image_address:
            data = client.get_model(data.id)
        return data


class ListModels(command.Lister):
    _description = _("List models")
    columns = ("Id", "Name", "Version", "Model Size")

    def get_parser(self, prog_name):
        parser = super(ListModels, self).get_parser(prog_name)
        parser.add_argument(
            "--limit",
            metavar="<limit>",
            type=int,
            help=_(
                "Maximum number of records returned on each page. "
                "The default value is 100. The recommended value ranges "
                "from 10 to 50."
            ),
        )
        parser.add_argument(
            "--offset",
            metavar="<offset>",
            type=int,
            help=_("Index of the page to be queried. Default value: 0."),
        )
        parser.add_argument(
            "--order",
            metavar="<order>",
            help=_(
                "Sorting order. The value can be asc or desc, "
                "indicating ascending or descending order. "
                "Default value: desc"
            ),
        )
        parser.add_argument(
            "--workspace-id",
            metavar="<workspace_id>",
            help=_(
                "ID of the workspace to which a service belongs. The "
                "default value is 0, indicating the default workspace."
            ),
        )
        parser.add_argument(
            "--sort-by",
            metavar="<sort_by>",
            help=_(
                "Sorting mode. The value can be create_at, model_"
                "version, or model_size. Default value: create_at."
            ),
        )
        parser.add_argument(
            "--name",
            metavar="<name>",
            help=_("Model name. Fuzzy match is supported."),
        )
        parser.add_argument(
            "--model-version",
            metavar="<model_version>",
            help=_("Model version."),
        )
        parser.add_argument(
            "--status", metavar="<status>", help=_("Model status.")
        )
        parser.add_argument(
            "--model-type",
            metavar="<model_type>",
            help=_(
                "Model type. The models of this type are queried. "
                "model_type and not_model_type are mutually exclusive "
                "and cannot co-exist."
            ),
        )
        parser.add_argument(
            "--not-model-type",
            metavar="<not_model_type>",
            help=_(
                "Model type. A list of models of types except "
                "for this type are queried."
            ),
        )
        parser.add_argument(
            "--description",
            metavar="<description>",
            help=_("Description. Fuzzy match is supported."),
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1
        args_list = (
            "name",
            "model_version",
            "status",
            "model_type",
            "not_model_type",
            "description",
            "offset",
            "limit",
            "sort_by",
            "order",
            "workspace_id",
        )
        query_params = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                query_params[arg] = val

        data = client.models(**query_params)

        table = (
            self.columns,
            (utils.get_dict_properties(s, self.columns) for s in data),
        )
        return table



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.vlb import TestVlb


class TestMember(TestVlb):

    def setUp(self):
        super(TestMember, self).setUp()
        self.create_network()
        self.create_server()
        self.create_load_balancer()
        self.create_listener()
        self.create_pool()
        self.create_member()

    def test_01_list_members(self):
        members = list(self.client.members(TestVlb.pool))
        self.assertGreaterEqual(len(members), 0)

    def test_02_get_member(self):
        member = self.client.get_member(TestVlb.member, TestVlb.pool)
        self.assertIsNotNone(member)

    def test_03_find_member(self):
        member = self.client.find_member(TestVlb.member.name, TestVlb.pool)
        self.assertIsNotNone(member)

    def test_04_update_member(self):
        new_weight = 20
        member = self.client.update_member(
            TestVlb.member,
            TestVlb.pool,
            weight=new_weight,
        )
        self.assertEqual(member['weight'], new_weight)

        # cleanup
        self.client.delete_member(
            TestVlb.member,
            TestVlb.pool
        )
        self.client.delete_pool(
            TestVlb.pool
        )
        self.client.delete_listener(
            TestVlb.listener
        )
        self.client.delete_load_balancer(
            TestVlb.load_balancer
        )
        self.net_client.delete_ip(
            TestVlb.load_balancer.floating_ips[0]['publicip_id']
        )
        self.delete_server()
        self.ecs_client.delete_keypair(TestVlb.keypair)

        self.addCleanup(self.destroy_network, TestVlb.network)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.apig import TestApiG


class TestAppCode(TestApiG):
    app = None
    code = None

    def setUp(self):
        super(TestAppCode, self).setUp()
        self.create_gateway()
        self.gateway_id = TestAppCode.gateway.id
        self.attrs = {
            "name": "app_demo",
            "remark": "Demo app"
        }
        TestAppCode.app = self.client.create_app(self.gateway_id, **self.attrs)
        self.addCleanup(
            self.client.delete_app,
            gateway=self.gateway_id,
            app=TestAppCode.app.id,
        )
        attrs = {
            "app_code": "GjOD3g80AABuuFeEJpVQADBlAjBh3UzC7W+gr4VJBB5BtJ4f"
                        "dVOQoSvoji3gFxUDb5pWBz9wUcw9+8/bFZ1B/4pq29wCMQC0"
                        "pQWX6zTndljDEl99As1pw+WntAU9xcq+ffagoH6zDpKUvdxV"
                        "6Ezj8LcCcPZN6BU="
        }
        TestAppCode.code = self.client.create_app_code(self.gateway_id,
                                                       TestAppCode.app.id,
                                                       **attrs)
        self.assertIsNotNone(TestAppCode.code.id)
        self.addCleanup(
            self.client.delete_app_code,
            gateway=self.gateway_id,
            app=TestAppCode.app.id,
            app_code=TestAppCode.code.id,
        )

    def test_generate_code(self):
        code = self.client.generate_app_code(gateway=TestAppCode.gateway.id,
                                             app=TestAppCode.app.id)
        self.assertIsNotNone(code.id)
        TestAppCode.code = code

    def test_get_code(self):
        code = self.client.get_app_code(gateway=TestAppCode.gateway.id,
                                        app=TestAppCode.app.id,
                                        app_code=TestAppCode.code.id)
        self.assertIsNotNone(code.app_code)

    def test_app_codes(self):
        codes = self.client.app_codes(gateway=TestAppCode.gateway.id,
                                      app=TestAppCode.app.id)
        self.assertEqual(len(list(codes)), 1)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy

from otcextensions.common.utils import extract_url_parts
from otcextensions.sdk.kms.v1 import data_key as _data_key
from otcextensions.sdk.kms.v1 import key as _key
from otcextensions.sdk.kms.v1 import misc as _misc


class Proxy(proxy.Proxy):

    skip_discovery = True

    def _extract_name(self, url, service_type=None, project_id=None):
        return extract_url_parts(url, project_id)

    def __init__(self, session, *args, **kwargs):
        super(Proxy, self).__init__(session=session, *args, **kwargs)
        self.additional_headers = {
            'Content-Type': 'application/json',
            'X-Language': 'en-us'
        }

    # ======== CMK Keys ========
    def keys(self, **query):
        """List all master keys.

        :param dict query: Keyword arguments which will be used to list keys.
            limit, marker, sequence, key_state are allowed.
            Key state can be:
            * 1 indicates that the CMK is waiting to be activated.
            * 2 indicates that the CMK is enabled.
            * 3 indicates that the CMK is disabled.
            * 4 indicates that the CMK is scheduled for deletion.

        :returns: a generator of
            (:class:`~otcextensions.sdk.kms.v1.key.Key`) instances
        """
        return self._list(_key.Key, **query)

    def create_key(self, **attrs):
        """Create master key

        :param dict attrs: Keyword arguments which will be used to overwrite a
            :class:`~otcextensions.sdk.kms.v1.key.Key`

        :returns: instance of :class:`~otcextensions.sdk.kms.v1.key.Key`
        """
        return self._create(
            _key.Key, prepend_key=False, **attrs
        )

    def get_key(self, key):
        """Describe a encrypt key by given key id or key object

        :param key: key id or an instance of
            :class:`~otcextensions.sdk.kms.v1.key.Key`
        :returns: instance of :class:`~otcextensions.sdk.kms.v1.key.Key`
        """
        return self._get(
            _key.Key, key,
        )

    def find_key(self, alias, ignore_missing=False):
        """Find a single key

        :param alias: The key alias
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the group does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent group.

        :returns: instance of :class:`~otcextensions.sdk.kms.v1.key.Key`
        """
        return self._find(
            _key.Key, alias,
            ignore_missing=ignore_missing,
        )

    def enable_key(self, key):
        """Enable a key

        :param key: key id or an instance of
            :class:`~otcextensions.sdk.kms.v1.key.Key`

        :returns: Updated instance of
            :class:`~otcextensions.sdk.kms.v1.key.Key`
        """
        key = self._get_resource(_key.Key, key)
        return key.enable(self)

    def disable_key(self, key):
        """Disable a key

        :param key: key id or an instance of
            :class:`~otcextensions.sdk.kms.v1.key.Key`

        :returns: Updated instance of
            :class:`~otcextensions.sdk.kms.v1.key.Key`
        """
        key = self._get_resource(_key.Key, key)
        return key.disable(self)

    def schedule_key_deletion(self, key, pending_days=7):
        """Schedule a key deletion

        :param key: key id or an instance of
            :class:`~otcextensions.sdk.kms.v1.key.Key`
        :param pending_days: Pending days before deletion, allow 7 to 1096
        :returns: Updated instance of
            :class:`~otcextensions.sdk.kms.v1.key.Key`
        """
        key = self._get_resource(_key.Key, key)
        return key.schedule_deletion(self, pending_days)

    def cancel_key_deletion(self, key):
        """Cancel a key deletion

        :param key: key id or an instance of
            :class:`~otcextensions.sdk.kms.v1.key.Key`
        :returns: Updated instance of
            :class:`~otcextensions.sdk.kms.v1.key.Key`
        """
        key = self._get_resource(_key.Key, key)
        return key.cancel_deletion(self)

    # ======== Data Keys ========
    def create_datakey(self, cmk, **attrs):
        """Create a data key

        :param cmk: key id or an instance of master key
            :class:`~otcextensions.sdk.kms.v1.key.Key`
        :param dict attrs: Keyword arguments which will be used to create a
            Data key.
            encryption_context, sequence are optional parameters.
        :returns: instance of
            :class:`~otcextensions.sdk.kms.v1.data_key.DataKey`
        """
        key = self._get_resource(_key.Key, cmk)
        attrs['key_id'] = key.id
        return self._create(
            _data_key.DataKey, prepend_key=False, **attrs
        )

    def create_datakey_wo_plain(self, cmk, **attrs):
        """Create a data key without plain text

        :param cmk: key id or an instance of
            :class:`~otcextensions.sdk.kms.v1.key.Key`
        :param dict attrs: Keyword arguments which will be used to create a
            Data key. encryption_context, sequence are optional parameters.
        :returns: instance of
            :class:`~otcextensions.sdk.kms.v1.data_key.DataKey`
        """
        key = self._get_resource(_key.Key, cmk)
        attrs['key_id'] = key.id
        res = _data_key.DataKey.new(**attrs)
        persist = res.create_wo_plain(
            self,
            prepend_key=False,
        )
        return persist

    def encrypt_datakey(self, datakey):
        """Encrypt a data key

        Requires `plain_text` to be filled with the hex key value.
        Populates `cipher_text` with the encrypted value.

        :param datakey: key id or an instance of
            :class:`~otcextensions.sdk.kms.v1.data_key.DataKey`
        :returns: instance of
            :class:`~otcextensions.sdk.kms.v1.data_key.DataKey`
        """
        key = self._get_resource(_data_key.DataKey, datakey)
        key.encrypt(self)
        return key

    # def decrypt_datakey(self, datakey):
    #     """Decrypt a data key
    #
    #     Requires `cipher_text` to be filled with value retrieved from
    #     :func:`~otcextensions.sdk.kms.v1.data_key.DataKey.encrypt` call.
    #     Populates `plain_text` attribute.
    #
    #     :param datakey: key id or an instance of
    #         :class:`~otcextensions.sdk.kms.v1.data_key.DataKey`
    #     :returns: update key instance
    #         :class:`~otcextensions.sdk.kms.v1.data_key.DataKey`
    #     """
    #     key = self._get_resource(_data_key.DataKey, datakey)
    #     key.decrypt(self)
    #     return key

    def decrypt_datakey(self, cmk, cipher_text, datakey_cipher_length):
        """Decrypt a data key

        :param cmk: key id or an instance of
            :class:`~otcextensions.sdk.kms.v1.key.Key`
        :param cipher_text: encrypted value retrieved from
            :func:`~otcextensions.sdk.kms.v1.data_key.DataKey.encrypt` call.
        :param datakey_cipher_length: datakey_cipher_length (expected value 64)

        :returns: decrypted key instance of
            :class:`~otcextensions.sdk.kms.v1.data_key.DataKey` with plain_text
            populated
        """
        cmk_key = self._get_resource(_key.Key, cmk)
        key_attrs = {
            'key_id': cmk_key.id,
            'cipher_text': cipher_text,
            'datakey_cipher_length': datakey_cipher_length
        }
        key = self._get_resource(_data_key.DataKey, value=None, **key_attrs)
        key.decrypt(self)
        return key

    # ======== Misc ========
    def generate_random(self, random_data_length=512):
        """Generate random data

        :param random_data_length: random data size in bits [0..512]
        :returns: instance of :class:`~otcextensions.sdk.kms.v1.random.Random`
        """
        return self._create(
            _misc.Random, prepend_key=False,
            random_data_length=random_data_length
        )

    def get_instance_number(self):
        """Get encrypt key instance total number

        :returns: instance of
            :class:`~otcextensions.sdk.kms.v1.key.InstanceNumber`
        """
        instance_num_obj = _misc.InstanceNumber()
        return instance_num_obj.fetch(self)

    def quotas(self):
        """List quota resources for KMS service

        :returns: A generator of
            :class:`~otcextensions.sdk.kms.v1.key.Quota` objects
        """
        return _misc.Quota.list(self)
